/* A Bison parser, made by GNU Bison 3.7.6.  */

/* Bison implementation for Yacc-like parsers in C

   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
   Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* C LALR(1) parser skeleton written by Richard Stallman, by
   simplifying the original so-called "semantic" parser.  */

/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
   especially those whose name start with YY_ or yy_.  They are
   private implementation details that can be changed or removed.  */

/* All symbols defined below should begin with yy or YY, to avoid
   infringing on user name space.  This should be done even for local
   variables, as they might otherwise be expanded by user macros.
   There are some unavoidable exceptions within include files to
   define necessary library symbols; they are noted "INFRINGES ON
   USER NAME SPACE" below.  */

/* Identify Bison output, and Bison version.  */
#define YYBISON 30706

/* Bison version string.  */
#define YYBISON_VERSION "3.7.6"

/* Skeleton name.  */
#define YYSKELETON_NAME "yacc.c"

/* Pure parsers.  */
#define YYPURE 0

/* Push parsers.  */
#define YYPUSH 0

/* Pull parsers.  */
#define YYPULL 1




/* First part of user prologue.  */
#line 17 "gram.y"

/*
 *  pmlogrewrite parser
 */
#include "pmapi.h"
#include "libpcp.h"
#include "logger.h"
#include <errno.h>
#include <assert.h>

#define PM_TEXT_TYPE_MASK (PM_TEXT_ONELINE | PM_TEXT_HELP)

static indomspec_t	*current_indomspec;
static int		current_star_indom;
static int		do_walk_indom;
static int		star_domain;

static metricspec_t	*current_metricspec;
static int		current_star_metric;
static int		star_cluster;
static int		do_walk_metric;
static int		output = OUTPUT_ALL;
static int		one_inst;
static char		*one_name;

static textspec_t	*current_textspec;
static int		do_walk_text;

static int		current_label_id;
static char *		current_label_name;
static char *		current_label_value;
static int		current_label_instance;
static char *		current_label_instance_name;
static labelspec_t	*current_labelspec;
static int		do_walk_label;

indomspec_t *
walk_indom(int mode)
{
    static indomspec_t	*ip;

    if (do_walk_indom) {
	if (mode == W_START)
	    ip = indom_root;
	else
	    ip = ip->i_next;
	while (ip != NULL && pmInDom_domain(ip->old_indom) != star_domain)
	    ip = ip->i_next;
    }
    else {
	if (mode == W_START)
	    ip = current_indomspec;
	else
	    ip = NULL;
    }

    return ip;
}

metricspec_t *
walk_metric(int mode, int flag, char *which, int dupok)
{
    static metricspec_t	*mp;

    if (do_walk_metric) {
	if (mode == W_START)
	    mp = metric_root;
	else
	    mp = mp->m_next;
	while (mp != NULL) {
	    if (pmID_domain(mp->old_desc.pmid) == star_domain &&
		(star_cluster == PM_ID_NULL || star_cluster == pmID_cluster(mp->old_desc.pmid)))
		break;
	    mp = mp->m_next;
	}
    }
    else {
	if (mode == W_START)
	    mp = current_metricspec;
	else
	    mp = NULL;
    }

    if (mp != NULL) {
	if (!dupok && (mp->flags & flag)) {
	    pmsprintf(mess, sizeof(mess), "Duplicate %s clause for metric %s", which, mp->old_name);
	    yyerror(mess);
	}
	if (flag != METRIC_DELETE) {
	    if (mp->flags & METRIC_DELETE) {
		pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted metric %s", which, mp->old_name);
		yyerror(mess);
	    }
	}
	else {
	    if (mp->flags & (~METRIC_DELETE)) {
		pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for metric %s", mp->old_name);
		yyerror(mess);
	    }
	}
    }

    return mp;
}

static const char *
textTypeStr(int type)
{
    switch(type)
	{
	case PM_TEXT_ONELINE:
	    return "one line";
	case PM_TEXT_HELP:
	    return "help";
	default:
	    break;
	}

    return "unknown";
}
 
textspec_t *
walk_text(int mode, int flag, char *which, int dupok)
{
    static textspec_t	*tp;

    if (do_walk_text) {
	if (mode == W_START)
	    tp = text_root;
	else
	    tp = tp->t_next;

	/* Only consider the active specs. */
	while (tp != NULL && (tp->flags & TEXT_ACTIVE) == 0)
	    tp = tp->t_next;
    }
    else {
	if (mode == W_START) {
	    tp = current_textspec;
	    if (tp)
		assert ((tp->flags & TEXT_ACTIVE));
	}
	else
	    tp = NULL;
    }

    if (tp != NULL) {
	tp->flags &= ~TEXT_ACTIVE;
	
	if (!dupok && (tp->flags & flag)) {
	    if ((tp->old_type & PM_TEXT_PMID)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for %s text for metric %s",
			  which, textTypeStr(tp->old_type), pmIDStr(tp->old_id));
	    }
	    else {
		assert((tp->old_type & PM_TEXT_INDOM));
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for %s text for indom %s",
			  which, textTypeStr(tp->old_type), pmInDomStr(tp->old_id));
	    }
	    yyerror(mess);
	}
	if (flag != TEXT_DELETE) {
	    if (tp->flags & TEXT_DELETE) {
		if ((tp->old_type & PM_TEXT_PMID)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted %s text for metric %s",
			      which, textTypeStr(tp->old_type), pmIDStr(tp->old_id));
		}
		else {
		    assert((tp->old_type & PM_TEXT_INDOM));
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted %s text for indom %s",
			      which, textTypeStr(tp->old_type), pmInDomStr(tp->old_id));
		}
		yyerror(mess);
	    }
	}
	else {
	    if (tp->flags & (~TEXT_DELETE)) {
		if ((tp->old_type & PM_TEXT_PMID)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for %s text for metric %s",
			      textTypeStr(tp->old_type), pmIDStr(tp->old_id));
		}
		else {
		    assert((tp->old_type & PM_TEXT_INDOM));
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for %s text for indom %s",
			      textTypeStr(tp->old_type), pmInDomStr(tp->old_id));
		}
		yyerror(mess);
	    }
	}
    }

    return tp;
}

labelspec_t *
walk_label(int mode, int flag, char *which, int dupok)
{
    static labelspec_t	*lp;

    if (do_walk_label > 1) {
	if (mode == W_START)
	    lp = label_root;
	else
	    lp = lp->l_next;

	/* Only consider the active specs. */
	while (lp != NULL && (lp->flags & LABEL_ACTIVE) == 0)
	    lp = lp->l_next;
    }
    else {
	if (mode == W_START) {
	    lp = current_labelspec;
	    if (lp)
		assert ((lp->flags & LABEL_ACTIVE));
	}
	else
	    lp = NULL;
    }

    if (lp != NULL) {
	lp->flags &= ~LABEL_ACTIVE;
	
	if (!dupok && (lp->flags & flag)) {
	    if ((lp->old_type & PM_LABEL_CONTEXT)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for context label",
			  which);
	    }
	    else if ((lp->old_type & PM_LABEL_DOMAIN)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for domain %d",
			  which, pmID_domain(lp->old_id));
	    }
	    else if ((lp->old_type & PM_LABEL_CLUSTER)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for cluster %d.%d",
			  which, pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
	    }
	    else if ((lp->old_type & PM_LABEL_ITEM)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for metric %s",
			  which, pmIDStr(lp->old_id));
	    }
	    else if ((lp->old_type & PM_LABEL_INDOM)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for indom %s",
			  which, pmInDomStr(lp->old_id));
	    }
	    else if ((lp->old_type & PM_LABEL_INSTANCES)) {
		pmsprintf(mess, sizeof(mess), "Duplicate %s clause for label for the instances of indom %s",
			  which, pmInDomStr(lp->old_id));
	    }
	    yyerror(mess);
	}
	if (flag != LABEL_DELETE) {
	    if (lp->flags & LABEL_DELETE) {
		if ((lp->old_type & PM_LABEL_CONTEXT)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted context label",
			      which);
		}
		else if ((lp->old_type & PM_LABEL_DOMAIN)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for domain %d",
			      which, pmID_domain(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_CLUSTER)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for cluster %d.%d",
			      which, pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_ITEM)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for metric %s",
			      which, pmIDStr(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_INDOM)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for indom %s",
			      which, pmInDomStr(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_INSTANCES)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting %s clause for deleted label for the instances of indom %s",
			      which, pmInDomStr(lp->old_id));
		}
		yyerror(mess);
	    }
	}
	else {
	    if (lp->flags & (~LABEL_DELETE)) {
		if ((lp->old_type & PM_LABEL_CONTEXT)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for context label");
		}
		else if ((lp->old_type & PM_LABEL_DOMAIN)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for domain %d",
			      pmID_domain(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_CLUSTER)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for cluster %d.%d",
			      pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_ITEM)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for metric %s",
			      pmIDStr(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_INDOM)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for indom %s",
			      pmInDomStr(lp->old_id));
		}
		else if ((lp->old_type & PM_LABEL_INSTANCES)) {
		    pmsprintf(mess, sizeof(mess), "Conflicting delete and other clauses for label for the instances of indom %s",
			      pmInDomStr(lp->old_id));
		}
		yyerror(mess);
	    }
	}
    }

    return lp;
}

void
deactivate_labels(void)
{
    labelspec_t	*lp;
    for (lp = walk_label(W_START, LABEL_ACTIVE, "active", 0); lp != NULL; lp = walk_label(W_NEXT, 0, "", 0)) {
	lp->flags &= ~LABEL_ACTIVE;
    }
}

void
new_context_label()
{
    labelspec_t	*lp;
    int		sts;
    char	buf[PM_MAXLABELJSONLEN];

    /*
     * Ignore the other change specs which were identified and
     * add the new label to the generic context label change spec.
     * Search for an existing one first.
     */
    deactivate_labels();
    for (lp = label_root; lp != NULL; lp = lp->l_next) {
	if (lp->old_type == PM_LABEL_CONTEXT) {
	    assert (lp->old_id == PM_ID_NULL);
	    if (lp->old_label == NULL)
		break;
	}
    }

    /* Create one if none exists. */
    if (lp == NULL)
	lp = create_label(PM_LABEL_CONTEXT, PM_ID_NULL, 0, NULL, NULL);

    /* Add the new label to the label change spec. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);
    if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_CONTEXT)) < 0) {
	pmsprintf(mess, sizeof(mess),
		  "Unable to add new context label %s: %s",
		  buf, pmErrStr(sts));
	yyerror(mess);
    }
    lp->new_labels->inst = 0;
    lp->flags |= LABEL_NEW;
}

void
new_domain_label(int domain)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    char		buf[PM_MAXLABELJSONLEN];

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashpmid;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmID_domain(node->key) != domain)
	    continue;

	/*
	 * A metric in this domain exists.
	 * Add the new label to the generic label change spec for this
	 * domain. Search for an existing one first.
	 */
	found = 1;
	for (lp = label_root; lp != NULL; lp = lp->l_next) {
	    if (lp->old_type != PM_LABEL_DOMAIN)
		continue;
	    if (lp->old_id != domain)
		continue;
	    if (lp->old_label == NULL)
		break;
	}

	/* Create one if none exists. */
	if (lp == NULL)
	    lp = create_label(PM_LABEL_DOMAIN, domain, 0, NULL, NULL);

	/* Add the new label to the label change spec. */
	if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_DOMAIN)) < 0) {
	    pmsprintf(mess, sizeof(mess),
		      "Unable to add new domain label %s: %s",
		      buf, pmErrStr(sts));
	    yyerror(mess);
	}
	lp->new_labels->inst = 0;
	lp->flags |= LABEL_NEW;

	/* We only need to find one metric in the specified domain. */
	break;
    }

    /* Did we find any clusters matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching domain for new label %s", buf);
	yywarn(mess);
    }
}

void
new_cluster_label(int cluster)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    char		buf[PM_MAXLABELJSONLEN] = "";

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashpmid;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmID_domain(node->key) != pmID_domain(cluster))
	    continue;
	if (! current_star_metric && 
	    pmID_cluster(node->key) != pmID_cluster(cluster))
	    continue;

	/*
	 * A metric in this cluster exists.
	 * Add the new label to the generic label change spec for the
	 * identified cluster. Search for an existing one first.
	 */
	found = 1;
	for (lp = label_root; lp != NULL; lp = lp->l_next) {
	    if (lp->old_type != PM_LABEL_CLUSTER)
		continue;
	    if (pmID_domain(lp->old_id) != pmID_domain((pmID)(node->key)))
		continue;
	    if (pmID_cluster(lp->old_id) != pmID_cluster((pmID)(node->key)))
		continue;
	    if (lp->old_label == NULL)
		break;
	}

	if (lp == NULL) {
	    pmID current_cluster =
		pmID_build(pmID_domain(node->key), pmID_cluster(node->key), 0);
	    lp = create_label(PM_LABEL_CLUSTER, current_cluster, 0, NULL, NULL);
	}

	/* Add the new label to the label change spec. */
	if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_CLUSTER)) < 0) {
	    pmsprintf(mess, sizeof(mess),
		      "Unable to add new cluster label %s: %s",
		      buf, pmErrStr(sts));
	    yyerror(mess);
	}
	lp->new_labels->inst = 0;
	lp->flags |= LABEL_NEW;

	/* Do we need to look for more clusters? */
	if (! current_star_metric)
	    break;
    }

    /* Did we find any clusters matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching cluster for new label %s", buf);
	yywarn(mess);
    }
}

void
new_item_label(int item)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    char		buf[PM_MAXLABELJSONLEN] = "";

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashpmid;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmID_domain(node->key) != pmID_domain(item))
	    continue;
	if (current_star_metric < 2) {
	    if (pmID_cluster(node->key) != pmID_cluster(item))
		continue;
	    if (! current_star_metric && (pmID)(node->key) != (pmID)item)
		continue;
	}

	/*
	 * A metric with this pmID exists.
	 * Add the new label to the generic label change spec for the
	 * identified item. Search for an existing one first.
	 */
	found = 1;
	for (lp = label_root; lp != NULL; lp = lp->l_next) {
	    if (lp->old_type != PM_LABEL_ITEM)
		continue;
	    if ((pmID)(lp->old_id) != (pmID)(node->key))
		continue;
	    if (lp->old_label == NULL)
		break;
	}

	if (lp == NULL)
	    lp = create_label(PM_LABEL_ITEM, (pmID)node->key, 0, NULL, NULL);

	/* Add the new label to the label change spec. */
	if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_ITEM)) < 0) {
	    pmsprintf(mess, sizeof(mess),
		      "Unable to add new metric item label %s: %s",
		      buf, pmErrStr(sts));
	    yyerror(mess);
	}
	lp->new_labels->inst = 0;
	lp->flags |= LABEL_NEW;

	/* Do we need to look for more items? */
	if (! current_star_metric)
	    break;
    }

    /* Did we find any items matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching metric for new label %s", buf);
	yywarn(mess);
    }
}

void
new_indom_label(int indom)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    char		buf[PM_MAXLABELJSONLEN] = "";

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashindom;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmInDom_domain(node->key) != pmInDom_domain(indom))
	    continue;
	if (! current_star_indom && 
	    pmInDom_serial(node->key) != pmInDom_serial(indom))
	    continue;

	/*
	 * A metric in this indom exists.
	 * Add the new label to the generic label change spec for the
	 * identified indom. Search for an existing one first.
	 */
	found = 1;
	for (lp = label_root; lp != NULL; lp = lp->l_next) {
	    if (lp->old_type != PM_LABEL_INDOM)
		continue;
	    if (pmInDom_domain(lp->old_id) != pmInDom_domain((pmInDom)(node->key)))
		continue;
	    if (pmInDom_serial(lp->old_id) != pmInDom_serial((pmInDom)(node->key)))
		continue;
	    if (lp->old_instance != -1)
		continue;
	    if (lp->old_label == NULL)
		break;
	}

	if (lp == NULL) {
	    pmInDom current_indom =
		pmInDom_build(pmInDom_domain(node->key), pmInDom_serial(node->key));
	    lp = create_label(PM_LABEL_INDOM, current_indom, 0, NULL, NULL);
	}

	/* Add the new label to the label change spec. */
	if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_INDOM)) < 0) {
	    pmsprintf(mess, sizeof(mess),
		      "Unable to add new indom label %s: %s",
		      buf, pmErrStr(sts));
	    yyerror(mess);
	}
	lp->new_labels->inst = 0;
	lp->flags |= LABEL_NEW;

	/* Do we need to look for more indoms? */
	if (! current_star_indom)
	    break;
    }

    /* Did we find any indoms matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching indom for new label %s", buf);
	yywarn(mess);
    }
}

void
new_indom_instance_label(int indom)
{
    labelspec_t		*lp;
    int			sts;
    int			found = 0;
    __pmContext		*ctxp;
    __pmHashCtl		*hcp;
    __pmHashNode	*node;
    __pmLogInDom	*idp;
    char		buf[PM_MAXLABELJSONLEN] = "";

    /*
     * Ignore the change specs which were previously identified and instead
     * search the metadata for metrics in the given domain.
     * Create or re-use the change record for each one found. 
     */
    deactivate_labels();

    ctxp = __pmHandleToPtr(pmWhichContext());
    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
    PM_UNLOCK(ctxp->c_lock);

    /* Prepare the JSON for the new label. */
    pmsprintf(buf, sizeof(buf), "{%s:%s}",
	      current_label_name, current_label_value);

    hcp = &ctxp->c_archctl->ac_log->hashindom;
    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
	 node != NULL;
	 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
	if (pmInDom_domain(node->key) != pmInDom_domain(indom))
	    continue;
	if (! current_star_indom && 
	    pmInDom_serial(node->key) != pmInDom_serial(indom))
	    continue;

	/*
	 * A metric in this indom exists.
	 * Add the new label to the generic label change specs for the
	 * identified indom instances. Search for existing ones first.
	 */
	for (idp = (__pmLogInDom *)node->data; idp != NULL; idp = idp->next) {
	    int indom_ix;
	    for (indom_ix = 0; indom_ix < idp->numinst; ++indom_ix) {
		int instance = idp->instlist[indom_ix];
		if (current_label_instance != -1 && instance != current_label_instance)
		    continue;
		found = 1;
		for (lp = label_root; lp != NULL; lp = lp->l_next) {
		    if (lp->old_type != PM_LABEL_INSTANCES)
			continue;
		    if (pmInDom_domain(lp->old_id) != pmInDom_domain((pmInDom)(node->key)))
			continue;
		    if (pmInDom_serial(lp->old_id) != pmInDom_serial((pmInDom)(node->key)))
			continue;
		    if (lp->old_instance != instance)
			continue;
		    if (lp->old_label == NULL)
			break;
		}

		if (lp == NULL) {
		    pmInDom current_indom =
			pmInDom_build(pmInDom_domain(node->key), pmInDom_serial(node->key));
		    lp = create_label(PM_LABEL_INSTANCES, current_indom,
				      instance, NULL, NULL);
		}

		/* Add the new label to the label change spec. */
		if ((sts = __pmAddLabels(&lp->new_labels, buf, PM_LABEL_INSTANCES)) < 0) {
		    pmsprintf(mess, sizeof(mess),
			      "Unable to add new indom instance label %s: %s",
			      buf, pmErrStr(sts));
		    yyerror(mess);
		}
		lp->new_labels->inst = instance;
		lp->flags |= LABEL_NEW;
	    }
	}

	/* Do we need to look for more indoms? */
	if (! current_star_indom)
	    break;
    }

    /* Did we find any indom instances matching the spec? */
    if (! found) {
	pmsprintf(mess, sizeof(mess),
		  "No matching indom instance for new label %s", buf);
	yywarn(mess);
    }
}


#line 880 "gram.tab.c"

# ifndef YY_CAST
#  ifdef __cplusplus
#   define YY_CAST(Type, Val) static_cast<Type> (Val)
#   define YY_REINTERPRET_CAST(Type, Val) reinterpret_cast<Type> (Val)
#  else
#   define YY_CAST(Type, Val) ((Type) (Val))
#   define YY_REINTERPRET_CAST(Type, Val) ((Type) (Val))
#  endif
# endif
# ifndef YY_NULLPTR
#  if defined __cplusplus
#   if 201103L <= __cplusplus
#    define YY_NULLPTR nullptr
#   else
#    define YY_NULLPTR 0
#   endif
#  else
#   define YY_NULLPTR ((void*)0)
#  endif
# endif

#include "gram.tab.h"
/* Symbol kind.  */
enum yysymbol_kind_t
{
  YYSYMBOL_YYEMPTY = -2,
  YYSYMBOL_YYEOF = 0,                      /* "end of file"  */
  YYSYMBOL_YYerror = 1,                    /* error  */
  YYSYMBOL_YYUNDEF = 2,                    /* "invalid token"  */
  YYSYMBOL_TOK_LBRACE = 3,                 /* TOK_LBRACE  */
  YYSYMBOL_TOK_RBRACE = 4,                 /* TOK_RBRACE  */
  YYSYMBOL_TOK_PLUS = 5,                   /* TOK_PLUS  */
  YYSYMBOL_TOK_MINUS = 6,                  /* TOK_MINUS  */
  YYSYMBOL_TOK_COLON = 7,                  /* TOK_COLON  */
  YYSYMBOL_TOK_COMMA = 8,                  /* TOK_COMMA  */
  YYSYMBOL_TOK_LPAREN = 9,                 /* TOK_LPAREN  */
  YYSYMBOL_TOK_RPAREN = 10,                /* TOK_RPAREN  */
  YYSYMBOL_TOK_ASSIGN = 11,                /* TOK_ASSIGN  */
  YYSYMBOL_TOK_GLOBAL = 12,                /* TOK_GLOBAL  */
  YYSYMBOL_TOK_INDOM = 13,                 /* TOK_INDOM  */
  YYSYMBOL_TOK_DUPLICATE = 14,             /* TOK_DUPLICATE  */
  YYSYMBOL_TOK_METRIC = 15,                /* TOK_METRIC  */
  YYSYMBOL_TOK_HOSTNAME = 16,              /* TOK_HOSTNAME  */
  YYSYMBOL_TOK_TIMEZONE = 17,              /* TOK_TIMEZONE  */
  YYSYMBOL_TOK_ZONEINFO = 18,              /* TOK_ZONEINFO  */
  YYSYMBOL_TOK_FEATURES = 19,              /* TOK_FEATURES  */
  YYSYMBOL_TOK_TIME = 20,                  /* TOK_TIME  */
  YYSYMBOL_TOK_NAME = 21,                  /* TOK_NAME  */
  YYSYMBOL_TOK_INST = 22,                  /* TOK_INST  */
  YYSYMBOL_TOK_INAME = 23,                 /* TOK_INAME  */
  YYSYMBOL_TOK_DELETE = 24,                /* TOK_DELETE  */
  YYSYMBOL_TOK_PMID = 25,                  /* TOK_PMID  */
  YYSYMBOL_TOK_NULL_INT = 26,              /* TOK_NULL_INT  */
  YYSYMBOL_TOK_TYPE = 27,                  /* TOK_TYPE  */
  YYSYMBOL_TOK_IF = 28,                    /* TOK_IF  */
  YYSYMBOL_TOK_SEM = 29,                   /* TOK_SEM  */
  YYSYMBOL_TOK_UNITS = 30,                 /* TOK_UNITS  */
  YYSYMBOL_TOK_OUTPUT = 31,                /* TOK_OUTPUT  */
  YYSYMBOL_TOK_RESCALE = 32,               /* TOK_RESCALE  */
  YYSYMBOL_TOK_TEXT = 33,                  /* TOK_TEXT  */
  YYSYMBOL_TOK_ONELINE = 34,               /* TOK_ONELINE  */
  YYSYMBOL_TOK_HELP = 35,                  /* TOK_HELP  */
  YYSYMBOL_TOK_TEXT_STAR = 36,             /* TOK_TEXT_STAR  */
  YYSYMBOL_TOK_LABEL = 37,                 /* TOK_LABEL  */
  YYSYMBOL_TOK_CONTEXT = 38,               /* TOK_CONTEXT  */
  YYSYMBOL_TOK_LABEL_STAR = 39,            /* TOK_LABEL_STAR  */
  YYSYMBOL_TOK_DOMAIN = 40,                /* TOK_DOMAIN  */
  YYSYMBOL_TOK_CLUSTER = 41,               /* TOK_CLUSTER  */
  YYSYMBOL_TOK_ITEM = 42,                  /* TOK_ITEM  */
  YYSYMBOL_TOK_INSTANCES = 43,             /* TOK_INSTANCES  */
  YYSYMBOL_TOK_INSTANCE = 44,              /* TOK_INSTANCE  */
  YYSYMBOL_TOK_NEW = 45,                   /* TOK_NEW  */
  YYSYMBOL_TOK_VALUE = 46,                 /* TOK_VALUE  */
  YYSYMBOL_TOK_BITS = 47,                  /* TOK_BITS  */
  YYSYMBOL_TOK_GNAME = 48,                 /* TOK_GNAME  */
  YYSYMBOL_TOK_NUMBER = 49,                /* TOK_NUMBER  */
  YYSYMBOL_TOK_STRING = 50,                /* TOK_STRING  */
  YYSYMBOL_TOK_TEXT_STRING = 51,           /* TOK_TEXT_STRING  */
  YYSYMBOL_TOK_HNAME = 52,                 /* TOK_HNAME  */
  YYSYMBOL_TOK_FLOAT = 53,                 /* TOK_FLOAT  */
  YYSYMBOL_TOK_JSON_STRING = 54,           /* TOK_JSON_STRING  */
  YYSYMBOL_TOK_JSON_NUMBER = 55,           /* TOK_JSON_NUMBER  */
  YYSYMBOL_TOK_JSON_TRUE = 56,             /* TOK_JSON_TRUE  */
  YYSYMBOL_TOK_JSON_FALSE = 57,            /* TOK_JSON_FALSE  */
  YYSYMBOL_TOK_JSON_NULL = 58,             /* TOK_JSON_NULL  */
  YYSYMBOL_TOK_INDOM_STAR = 59,            /* TOK_INDOM_STAR  */
  YYSYMBOL_TOK_PMID_INT = 60,              /* TOK_PMID_INT  */
  YYSYMBOL_TOK_PMID_STAR = 61,             /* TOK_PMID_STAR  */
  YYSYMBOL_TOK_TYPE_NAME = 62,             /* TOK_TYPE_NAME  */
  YYSYMBOL_TOK_SEM_NAME = 63,              /* TOK_SEM_NAME  */
  YYSYMBOL_TOK_SPACE_NAME = 64,            /* TOK_SPACE_NAME  */
  YYSYMBOL_TOK_TIME_NAME = 65,             /* TOK_TIME_NAME  */
  YYSYMBOL_TOK_COUNT_NAME = 66,            /* TOK_COUNT_NAME  */
  YYSYMBOL_TOK_OUTPUT_TYPE = 67,           /* TOK_OUTPUT_TYPE  */
  YYSYMBOL_YYACCEPT = 68,                  /* $accept  */
  YYSYMBOL_config = 69,                    /* config  */
  YYSYMBOL_speclist = 70,                  /* speclist  */
  YYSYMBOL_spec = 71,                      /* spec  */
  YYSYMBOL_globalspec = 72,                /* globalspec  */
  YYSYMBOL_globaloptlist = 73,             /* globaloptlist  */
  YYSYMBOL_globalopt = 74,                 /* globalopt  */
  YYSYMBOL_feature_bits = 75,              /* feature_bits  */
  YYSYMBOL_bits_list = 76,                 /* bits_list  */
  YYSYMBOL_hname = 77,                     /* hname  */
  YYSYMBOL_signnumber = 78,                /* signnumber  */
  YYSYMBOL_number = 79,                    /* number  */
  YYSYMBOL_float = 80,                     /* float  */
  YYSYMBOL_signtime = 81,                  /* signtime  */
  YYSYMBOL_time = 82,                      /* time  */
  YYSYMBOL_indomspec = 83,                 /* indomspec  */
  YYSYMBOL_84_1 = 84,                      /* $@1  */
  YYSYMBOL_indom_int = 85,                 /* indom_int  */
  YYSYMBOL_optindomopt = 86,               /* optindomopt  */
  YYSYMBOL_indomoptlist = 87,              /* indomoptlist  */
  YYSYMBOL_indomopt = 88,                  /* indomopt  */
  YYSYMBOL_duplicateopt = 89,              /* duplicateopt  */
  YYSYMBOL_metricspec = 90,                /* metricspec  */
  YYSYMBOL_91_2 = 91,                      /* $@2  */
  YYSYMBOL_pmid_or_name = 92,              /* pmid_or_name  */
  YYSYMBOL_pmid_int = 93,                  /* pmid_int  */
  YYSYMBOL_optmetricoptlist = 94,          /* optmetricoptlist  */
  YYSYMBOL_metricoptlist = 95,             /* metricoptlist  */
  YYSYMBOL_metricopt = 96,                 /* metricopt  */
  YYSYMBOL_null_or_indom = 97,             /* null_or_indom  */
  YYSYMBOL_pick = 98,                      /* pick  */
  YYSYMBOL_rescaleopt = 99,                /* rescaleopt  */
  YYSYMBOL_textspec = 100,                 /* textspec  */
  YYSYMBOL_textmetricorindomspec = 101,    /* textmetricorindomspec  */
  YYSYMBOL_textmetricspec = 102,           /* textmetricspec  */
  YYSYMBOL_103_3 = 103,                    /* $@3  */
  YYSYMBOL_opttexttypes = 104,             /* opttexttypes  */
  YYSYMBOL_texttypes = 105,                /* texttypes  */
  YYSYMBOL_texttype = 106,                 /* texttype  */
  YYSYMBOL_opttextvalue = 107,             /* opttextvalue  */
  YYSYMBOL_opttextmetricoptlist = 108,     /* opttextmetricoptlist  */
  YYSYMBOL_textmetricoptlist = 109,        /* textmetricoptlist  */
  YYSYMBOL_textmetricopt = 110,            /* textmetricopt  */
  YYSYMBOL_textstring = 111,               /* textstring  */
  YYSYMBOL_textindomspec = 112,            /* textindomspec  */
  YYSYMBOL_113_4 = 113,                    /* $@4  */
  YYSYMBOL_opttextindomoptlist = 114,      /* opttextindomoptlist  */
  YYSYMBOL_textindomoptlist = 115,         /* textindomoptlist  */
  YYSYMBOL_textindomopt = 116,             /* textindomopt  */
  YYSYMBOL_labelspec = 117,                /* labelspec  */
  YYSYMBOL_labelcontextormetricorindomspec = 118, /* labelcontextormetricorindomspec  */
  YYSYMBOL_labelcontextspec = 119,         /* labelcontextspec  */
  YYSYMBOL_120_5 = 120,                    /* $@5  */
  YYSYMBOL_optlabeldetails = 121,          /* optlabeldetails  */
  YYSYMBOL_jsonname = 122,                 /* jsonname  */
  YYSYMBOL_optlabelvalue = 123,            /* optlabelvalue  */
  YYSYMBOL_jsonvalue = 124,                /* jsonvalue  */
  YYSYMBOL_jsonnumber = 125,               /* jsonnumber  */
  YYSYMBOL_optlabelcontextoptlist = 126,   /* optlabelcontextoptlist  */
  YYSYMBOL_labelcontextoptlist = 127,      /* labelcontextoptlist  */
  YYSYMBOL_labelcontextopt = 128,          /* labelcontextopt  */
  YYSYMBOL_newlabelspec = 129,             /* newlabelspec  */
  YYSYMBOL_labeldomainspec = 130,          /* labeldomainspec  */
  YYSYMBOL_131_6 = 131,                    /* $@6  */
  YYSYMBOL_pmid_domain = 132,              /* pmid_domain  */
  YYSYMBOL_optlabeldomainoptlist = 133,    /* optlabeldomainoptlist  */
  YYSYMBOL_labeldomainoptlist = 134,       /* labeldomainoptlist  */
  YYSYMBOL_labeldomainopt = 135,           /* labeldomainopt  */
  YYSYMBOL_labelclusterspec = 136,         /* labelclusterspec  */
  YYSYMBOL_137_7 = 137,                    /* $@7  */
  YYSYMBOL_pmid_cluster = 138,             /* pmid_cluster  */
  YYSYMBOL_optlabelclusteroptlist = 139,   /* optlabelclusteroptlist  */
  YYSYMBOL_labelclusteroptlist = 140,      /* labelclusteroptlist  */
  YYSYMBOL_labelclusteropt = 141,          /* labelclusteropt  */
  YYSYMBOL_labelitemspec = 142,            /* labelitemspec  */
  YYSYMBOL_143_8 = 143,                    /* $@8  */
  YYSYMBOL_optlabelitemoptlist = 144,      /* optlabelitemoptlist  */
  YYSYMBOL_labelitemoptlist = 145,         /* labelitemoptlist  */
  YYSYMBOL_labelitemopt = 146,             /* labelitemopt  */
  YYSYMBOL_labelindomspec = 147,           /* labelindomspec  */
  YYSYMBOL_148_9 = 148,                    /* $@9  */
  YYSYMBOL_optlabelindomoptlist = 149,     /* optlabelindomoptlist  */
  YYSYMBOL_labelindomoptlist = 150,        /* labelindomoptlist  */
  YYSYMBOL_labelindomopt = 151,            /* labelindomopt  */
  YYSYMBOL_labelinstancesspec = 152,       /* labelinstancesspec  */
  YYSYMBOL_153_10 = 153,                   /* $@10  */
  YYSYMBOL_optinstancelabeldetails = 154,  /* optinstancelabeldetails  */
  YYSYMBOL_optlabelinstancesoptlist = 155, /* optlabelinstancesoptlist  */
  YYSYMBOL_labelinstancesoptlist = 156,    /* labelinstancesoptlist  */
  YYSYMBOL_labelinstancesopt = 157         /* labelinstancesopt  */
};
typedef enum yysymbol_kind_t yysymbol_kind_t;




#ifdef short
# undef short
#endif

/* On compilers that do not define __PTRDIFF_MAX__ etc., make sure
   <limits.h> and (if available) <stdint.h> are included
   so that the code can choose integer types of a good width.  */

#ifndef __PTRDIFF_MAX__
# include <limits.h> /* INFRINGES ON USER NAME SPACE */
# if defined __STDC_VERSION__ && 199901 <= __STDC_VERSION__
#  include <stdint.h> /* INFRINGES ON USER NAME SPACE */
#  define YY_STDINT_H
# endif
#endif

/* Narrow types that promote to a signed type and that can represent a
   signed or unsigned integer of at least N bits.  In tables they can
   save space and decrease cache pressure.  Promoting to a signed type
   helps avoid bugs in integer arithmetic.  */

#ifdef __INT_LEAST8_MAX__
typedef __INT_LEAST8_TYPE__ yytype_int8;
#elif defined YY_STDINT_H
typedef int_least8_t yytype_int8;
#else
typedef signed char yytype_int8;
#endif

#ifdef __INT_LEAST16_MAX__
typedef __INT_LEAST16_TYPE__ yytype_int16;
#elif defined YY_STDINT_H
typedef int_least16_t yytype_int16;
#else
typedef short yytype_int16;
#endif

/* Work around bug in HP-UX 11.23, which defines these macros
   incorrectly for preprocessor constants.  This workaround can likely
   be removed in 2023, as HPE has promised support for HP-UX 11.23
   (aka HP-UX 11i v2) only through the end of 2022; see Table 2 of
   <https://h20195.www2.hpe.com/V2/getpdf.aspx/4AA4-7673ENW.pdf>.  */
#ifdef __hpux
# undef UINT_LEAST8_MAX
# undef UINT_LEAST16_MAX
# define UINT_LEAST8_MAX 255
# define UINT_LEAST16_MAX 65535
#endif

#if defined __UINT_LEAST8_MAX__ && __UINT_LEAST8_MAX__ <= __INT_MAX__
typedef __UINT_LEAST8_TYPE__ yytype_uint8;
#elif (!defined __UINT_LEAST8_MAX__ && defined YY_STDINT_H \
       && UINT_LEAST8_MAX <= INT_MAX)
typedef uint_least8_t yytype_uint8;
#elif !defined __UINT_LEAST8_MAX__ && UCHAR_MAX <= INT_MAX
typedef unsigned char yytype_uint8;
#else
typedef short yytype_uint8;
#endif

#if defined __UINT_LEAST16_MAX__ && __UINT_LEAST16_MAX__ <= __INT_MAX__
typedef __UINT_LEAST16_TYPE__ yytype_uint16;
#elif (!defined __UINT_LEAST16_MAX__ && defined YY_STDINT_H \
       && UINT_LEAST16_MAX <= INT_MAX)
typedef uint_least16_t yytype_uint16;
#elif !defined __UINT_LEAST16_MAX__ && USHRT_MAX <= INT_MAX
typedef unsigned short yytype_uint16;
#else
typedef int yytype_uint16;
#endif

#ifndef YYPTRDIFF_T
# if defined __PTRDIFF_TYPE__ && defined __PTRDIFF_MAX__
#  define YYPTRDIFF_T __PTRDIFF_TYPE__
#  define YYPTRDIFF_MAXIMUM __PTRDIFF_MAX__
# elif defined PTRDIFF_MAX
#  ifndef ptrdiff_t
#   include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  endif
#  define YYPTRDIFF_T ptrdiff_t
#  define YYPTRDIFF_MAXIMUM PTRDIFF_MAX
# else
#  define YYPTRDIFF_T long
#  define YYPTRDIFF_MAXIMUM LONG_MAX
# endif
#endif

#ifndef YYSIZE_T
# ifdef __SIZE_TYPE__
#  define YYSIZE_T __SIZE_TYPE__
# elif defined size_t
#  define YYSIZE_T size_t
# elif defined __STDC_VERSION__ && 199901 <= __STDC_VERSION__
#  include <stddef.h> /* INFRINGES ON USER NAME SPACE */
#  define YYSIZE_T size_t
# else
#  define YYSIZE_T unsigned
# endif
#endif

#define YYSIZE_MAXIMUM                                  \
  YY_CAST (YYPTRDIFF_T,                                 \
           (YYPTRDIFF_MAXIMUM < YY_CAST (YYSIZE_T, -1)  \
            ? YYPTRDIFF_MAXIMUM                         \
            : YY_CAST (YYSIZE_T, -1)))

#define YYSIZEOF(X) YY_CAST (YYPTRDIFF_T, sizeof (X))


/* Stored state numbers (used for stacks). */
typedef yytype_int16 yy_state_t;

/* State numbers in computations.  */
typedef int yy_state_fast_t;

#ifndef YY_
# if defined YYENABLE_NLS && YYENABLE_NLS
#  if ENABLE_NLS
#   include <libintl.h> /* INFRINGES ON USER NAME SPACE */
#   define YY_(Msgid) dgettext ("bison-runtime", Msgid)
#  endif
# endif
# ifndef YY_
#  define YY_(Msgid) Msgid
# endif
#endif


#ifndef YY_ATTRIBUTE_PURE
# if defined __GNUC__ && 2 < __GNUC__ + (96 <= __GNUC_MINOR__)
#  define YY_ATTRIBUTE_PURE __attribute__ ((__pure__))
# else
#  define YY_ATTRIBUTE_PURE
# endif
#endif

#ifndef YY_ATTRIBUTE_UNUSED
# if defined __GNUC__ && 2 < __GNUC__ + (7 <= __GNUC_MINOR__)
#  define YY_ATTRIBUTE_UNUSED __attribute__ ((__unused__))
# else
#  define YY_ATTRIBUTE_UNUSED
# endif
#endif

/* Suppress unused-variable warnings by "using" E.  */
#if ! defined lint || defined __GNUC__
# define YY_USE(E) ((void) (E))
#else
# define YY_USE(E) /* empty */
#endif

#if defined __GNUC__ && ! defined __ICC && 407 <= __GNUC__ * 100 + __GNUC_MINOR__
/* Suppress an incorrect diagnostic about yylval being uninitialized.  */
# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN                            \
    _Pragma ("GCC diagnostic push")                                     \
    _Pragma ("GCC diagnostic ignored \"-Wuninitialized\"")              \
    _Pragma ("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
# define YY_IGNORE_MAYBE_UNINITIALIZED_END      \
    _Pragma ("GCC diagnostic pop")
#else
# define YY_INITIAL_VALUE(Value) Value
#endif
#ifndef YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
# define YY_IGNORE_MAYBE_UNINITIALIZED_END
#endif
#ifndef YY_INITIAL_VALUE
# define YY_INITIAL_VALUE(Value) /* Nothing. */
#endif

#if defined __cplusplus && defined __GNUC__ && ! defined __ICC && 6 <= __GNUC__
# define YY_IGNORE_USELESS_CAST_BEGIN                          \
    _Pragma ("GCC diagnostic push")                            \
    _Pragma ("GCC diagnostic ignored \"-Wuseless-cast\"")
# define YY_IGNORE_USELESS_CAST_END            \
    _Pragma ("GCC diagnostic pop")
#endif
#ifndef YY_IGNORE_USELESS_CAST_BEGIN
# define YY_IGNORE_USELESS_CAST_BEGIN
# define YY_IGNORE_USELESS_CAST_END
#endif


#define YY_ASSERT(E) ((void) (0 && (E)))

#if !defined yyoverflow

/* The parser invokes alloca or malloc; define the necessary symbols.  */

# ifdef YYSTACK_USE_ALLOCA
#  if YYSTACK_USE_ALLOCA
#   ifdef __GNUC__
#    define YYSTACK_ALLOC __builtin_alloca
#   elif defined __BUILTIN_VA_ARG_INCR
#    include <alloca.h> /* INFRINGES ON USER NAME SPACE */
#   elif defined _AIX
#    define YYSTACK_ALLOC __alloca
#   elif defined _MSC_VER
#    include <malloc.h> /* INFRINGES ON USER NAME SPACE */
#    define alloca _alloca
#   else
#    define YYSTACK_ALLOC alloca
#    if ! defined _ALLOCA_H && ! defined EXIT_SUCCESS
#     include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
      /* Use EXIT_SUCCESS as a witness for stdlib.h.  */
#     ifndef EXIT_SUCCESS
#      define EXIT_SUCCESS 0
#     endif
#    endif
#   endif
#  endif
# endif

# ifdef YYSTACK_ALLOC
   /* Pacify GCC's 'empty if-body' warning.  */
#  define YYSTACK_FREE(Ptr) do { /* empty */; } while (0)
#  ifndef YYSTACK_ALLOC_MAXIMUM
    /* The OS might guarantee only one guard page at the bottom of the stack,
       and a page size can be as small as 4096 bytes.  So we cannot safely
       invoke alloca (N) if N exceeds 4096.  Use a slightly smaller number
       to allow for a few compiler-allocated temporary stack slots.  */
#   define YYSTACK_ALLOC_MAXIMUM 4032 /* reasonable circa 2006 */
#  endif
# else
#  define YYSTACK_ALLOC YYMALLOC
#  define YYSTACK_FREE YYFREE
#  ifndef YYSTACK_ALLOC_MAXIMUM
#   define YYSTACK_ALLOC_MAXIMUM YYSIZE_MAXIMUM
#  endif
#  if (defined __cplusplus && ! defined EXIT_SUCCESS \
       && ! ((defined YYMALLOC || defined malloc) \
             && (defined YYFREE || defined free)))
#   include <stdlib.h> /* INFRINGES ON USER NAME SPACE */
#   ifndef EXIT_SUCCESS
#    define EXIT_SUCCESS 0
#   endif
#  endif
#  ifndef YYMALLOC
#   define YYMALLOC malloc
#   if ! defined malloc && ! defined EXIT_SUCCESS
void *malloc (YYSIZE_T); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
#  ifndef YYFREE
#   define YYFREE free
#   if ! defined free && ! defined EXIT_SUCCESS
void free (void *); /* INFRINGES ON USER NAME SPACE */
#   endif
#  endif
# endif
#endif /* !defined yyoverflow */

#if (! defined yyoverflow \
     && (! defined __cplusplus \
         || (defined YYSTYPE_IS_TRIVIAL && YYSTYPE_IS_TRIVIAL)))

/* A type that is properly aligned for any stack member.  */
union yyalloc
{
  yy_state_t yyss_alloc;
  YYSTYPE yyvs_alloc;
};

/* The size of the maximum gap between one aligned stack and the next.  */
# define YYSTACK_GAP_MAXIMUM (YYSIZEOF (union yyalloc) - 1)

/* The size of an array large to enough to hold all stacks, each with
   N elements.  */
# define YYSTACK_BYTES(N) \
     ((N) * (YYSIZEOF (yy_state_t) + YYSIZEOF (YYSTYPE)) \
      + YYSTACK_GAP_MAXIMUM)

# define YYCOPY_NEEDED 1

/* Relocate STACK from its old location to the new one.  The
   local variables YYSIZE and YYSTACKSIZE give the old and new number of
   elements in the stack, and YYPTR gives the new location of the
   stack.  Advance YYPTR to a properly aligned location for the next
   stack.  */
# define YYSTACK_RELOCATE(Stack_alloc, Stack)                           \
    do                                                                  \
      {                                                                 \
        YYPTRDIFF_T yynewbytes;                                         \
        YYCOPY (&yyptr->Stack_alloc, Stack, yysize);                    \
        Stack = &yyptr->Stack_alloc;                                    \
        yynewbytes = yystacksize * YYSIZEOF (*Stack) + YYSTACK_GAP_MAXIMUM; \
        yyptr += yynewbytes / YYSIZEOF (*yyptr);                        \
      }                                                                 \
    while (0)

#endif

#if defined YYCOPY_NEEDED && YYCOPY_NEEDED
/* Copy COUNT objects from SRC to DST.  The source and destination do
   not overlap.  */
# ifndef YYCOPY
#  if defined __GNUC__ && 1 < __GNUC__
#   define YYCOPY(Dst, Src, Count) \
      __builtin_memcpy (Dst, Src, YY_CAST (YYSIZE_T, (Count)) * sizeof (*(Src)))
#  else
#   define YYCOPY(Dst, Src, Count)              \
      do                                        \
        {                                       \
          YYPTRDIFF_T yyi;                      \
          for (yyi = 0; yyi < (Count); yyi++)   \
            (Dst)[yyi] = (Src)[yyi];            \
        }                                       \
      while (0)
#  endif
# endif
#endif /* !YYCOPY_NEEDED */

/* YYFINAL -- State number of the termination state.  */
#define YYFINAL  41
/* YYLAST -- Last index in YYTABLE.  */
#define YYLAST   345

/* YYNTOKENS -- Number of terminals.  */
#define YYNTOKENS  68
/* YYNNTS -- Number of nonterminals.  */
#define YYNNTS  90
/* YYNRULES -- Number of rules.  */
#define YYNRULES  246
/* YYNSTATES -- Number of states.  */
#define YYNSTATES  360

/* YYMAXUTOK -- Last valid token kind.  */
#define YYMAXUTOK   322


/* YYTRANSLATE(TOKEN-NUM) -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex, with out-of-bounds checking.  */
#define YYTRANSLATE(YYX)                                \
  (0 <= (YYX) && (YYX) <= YYMAXUTOK                     \
   ? YY_CAST (yysymbol_kind_t, yytranslate[YYX])        \
   : YYSYMBOL_YYUNDEF)

/* YYTRANSLATE[TOKEN-NUM] -- Symbol number corresponding to TOKEN-NUM
   as returned by yylex.  */
static const yytype_int8 yytranslate[] =
{
       0,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
       2,     2,     2,     2,     2,     2,     1,     2,     3,     4,
       5,     6,     7,     8,     9,    10,    11,    12,    13,    14,
      15,    16,    17,    18,    19,    20,    21,    22,    23,    24,
      25,    26,    27,    28,    29,    30,    31,    32,    33,    34,
      35,    36,    37,    38,    39,    40,    41,    42,    43,    44,
      45,    46,    47,    48,    49,    50,    51,    52,    53,    54,
      55,    56,    57,    58,    59,    60,    61,    62,    63,    64,
      65,    66,    67
};

#if YYDEBUG
  /* YYRLINE[YYN] -- Source line where rule number YYN was defined.  */
static const yytype_int16 yyrline[] =
{
       0,   900,   900,   903,   904,   907,   908,   909,   910,   911,
     914,   915,   918,   919,   922,   941,   960,   983,  1005,  1021,
    1026,  1031,  1036,  1041,  1046,  1051,  1056,  1061,  1066,  1073,
    1079,  1083,  1088,  1093,  1100,  1111,  1131,  1132,  1133,  1134,
    1137,  1142,  1147,  1154,  1161,  1168,  1169,  1170,  1173,  1186,
    1198,  1211,  1223,  1232,  1244,  1243,  1276,  1283,  1305,  1319,
    1320,  1323,  1324,  1327,  1352,  1362,  1371,  1379,  1387,  1392,
    1397,  1402,  1407,  1412,  1417,  1422,  1429,  1430,  1434,  1433,
    1476,  1483,  1484,  1502,  1530,  1560,  1561,  1564,  1565,  1568,
    1592,  1616,  1644,  1675,  1749,  1766,  1812,  1819,  1824,  1829,
    1834,  1839,  1844,  1849,  1854,  1859,  1864,  1869,  1876,  1877,
    1883,  1889,  1895,  1899,  1904,  1907,  1909,  1912,  1915,  1916,
    1920,  1919,  2019,  2022,  2025,  2027,  2031,  2033,  2035,  2039,
    2042,  2045,  2047,  2055,  2056,  2059,  2066,  2095,  2144,  2146,
    2151,  2150,  2243,  2245,  2253,  2254,  2257,  2264,  2293,  2321,
    2324,  2325,  2326,  2327,  2328,  2329,  2333,  2332,  2387,  2389,
    2392,  2398,  2400,  2407,  2409,  2412,  2415,  2417,  2419,  2421,
    2423,  2427,  2432,  2434,  2439,  2443,  2445,  2450,  2451,  2454,
    2461,  2485,  2509,  2514,  2530,  2529,  2589,  2609,  2611,  2616,
    2617,  2620,  2627,  2653,  2677,  2701,  2708,  2707,  2786,  2809,
    2829,  2831,  2836,  2837,  2840,  2847,  2873,  2897,  2921,  2928,
    2927,  3017,  3019,  3024,  3025,  3028,  3035,  3063,  3087,  3111,
    3118,  3117,  3200,  3202,  3207,  3208,  3211,  3218,  3244,  3268,
    3292,  3299,  3298,  3383,  3388,  3394,  3400,  3408,  3410,  3415,
    3416,  3419,  3426,  3452,  3477,  3501,  3525
};
#endif

/** Accessing symbol of state STATE.  */
#define YY_ACCESSING_SYMBOL(State) YY_CAST (yysymbol_kind_t, yystos[State])

#if YYDEBUG || 0
/* The user-facing name of the symbol whose (internal) number is
   YYSYMBOL.  No bounds checking.  */
static const char *yysymbol_name (yysymbol_kind_t yysymbol) YY_ATTRIBUTE_UNUSED;

/* YYTNAME[SYMBOL-NUM] -- String name of the symbol SYMBOL-NUM.
   First, the terminals, then, starting at YYNTOKENS, nonterminals.  */
static const char *const yytname[] =
{
  "\"end of file\"", "error", "\"invalid token\"", "TOK_LBRACE",
  "TOK_RBRACE", "TOK_PLUS", "TOK_MINUS", "TOK_COLON", "TOK_COMMA",
  "TOK_LPAREN", "TOK_RPAREN", "TOK_ASSIGN", "TOK_GLOBAL", "TOK_INDOM",
  "TOK_DUPLICATE", "TOK_METRIC", "TOK_HOSTNAME", "TOK_TIMEZONE",
  "TOK_ZONEINFO", "TOK_FEATURES", "TOK_TIME", "TOK_NAME", "TOK_INST",
  "TOK_INAME", "TOK_DELETE", "TOK_PMID", "TOK_NULL_INT", "TOK_TYPE",
  "TOK_IF", "TOK_SEM", "TOK_UNITS", "TOK_OUTPUT", "TOK_RESCALE",
  "TOK_TEXT", "TOK_ONELINE", "TOK_HELP", "TOK_TEXT_STAR", "TOK_LABEL",
  "TOK_CONTEXT", "TOK_LABEL_STAR", "TOK_DOMAIN", "TOK_CLUSTER", "TOK_ITEM",
  "TOK_INSTANCES", "TOK_INSTANCE", "TOK_NEW", "TOK_VALUE", "TOK_BITS",
  "TOK_GNAME", "TOK_NUMBER", "TOK_STRING", "TOK_TEXT_STRING", "TOK_HNAME",
  "TOK_FLOAT", "TOK_JSON_STRING", "TOK_JSON_NUMBER", "TOK_JSON_TRUE",
  "TOK_JSON_FALSE", "TOK_JSON_NULL", "TOK_INDOM_STAR", "TOK_PMID_INT",
  "TOK_PMID_STAR", "TOK_TYPE_NAME", "TOK_SEM_NAME", "TOK_SPACE_NAME",
  "TOK_TIME_NAME", "TOK_COUNT_NAME", "TOK_OUTPUT_TYPE", "$accept",
  "config", "speclist", "spec", "globalspec", "globaloptlist", "globalopt",
  "feature_bits", "bits_list", "hname", "signnumber", "number", "float",
  "signtime", "time", "indomspec", "$@1", "indom_int", "optindomopt",
  "indomoptlist", "indomopt", "duplicateopt", "metricspec", "$@2",
  "pmid_or_name", "pmid_int", "optmetricoptlist", "metricoptlist",
  "metricopt", "null_or_indom", "pick", "rescaleopt", "textspec",
  "textmetricorindomspec", "textmetricspec", "$@3", "opttexttypes",
  "texttypes", "texttype", "opttextvalue", "opttextmetricoptlist",
  "textmetricoptlist", "textmetricopt", "textstring", "textindomspec",
  "$@4", "opttextindomoptlist", "textindomoptlist", "textindomopt",
  "labelspec", "labelcontextormetricorindomspec", "labelcontextspec",
  "$@5", "optlabeldetails", "jsonname", "optlabelvalue", "jsonvalue",
  "jsonnumber", "optlabelcontextoptlist", "labelcontextoptlist",
  "labelcontextopt", "newlabelspec", "labeldomainspec", "$@6",
  "pmid_domain", "optlabeldomainoptlist", "labeldomainoptlist",
  "labeldomainopt", "labelclusterspec", "$@7", "pmid_cluster",
  "optlabelclusteroptlist", "labelclusteroptlist", "labelclusteropt",
  "labelitemspec", "$@8", "optlabelitemoptlist", "labelitemoptlist",
  "labelitemopt", "labelindomspec", "$@9", "optlabelindomoptlist",
  "labelindomoptlist", "labelindomopt", "labelinstancesspec", "$@10",
  "optinstancelabeldetails", "optlabelinstancesoptlist",
  "labelinstancesoptlist", "labelinstancesopt", YY_NULLPTR
};

static const char *
yysymbol_name (yysymbol_kind_t yysymbol)
{
  return yytname[yysymbol];
}
#endif

#ifdef YYPRINT
/* YYTOKNUM[NUM] -- (External) token number corresponding to the
   (internal) symbol number NUM (which must be that of a token).  */
static const yytype_int16 yytoknum[] =
{
       0,   256,   257,   258,   259,   260,   261,   262,   263,   264,
     265,   266,   267,   268,   269,   270,   271,   272,   273,   274,
     275,   276,   277,   278,   279,   280,   281,   282,   283,   284,
     285,   286,   287,   288,   289,   290,   291,   292,   293,   294,
     295,   296,   297,   298,   299,   300,   301,   302,   303,   304,
     305,   306,   307,   308,   309,   310,   311,   312,   313,   314,
     315,   316,   317,   318,   319,   320,   321,   322
};
#endif

#define YYPACT_NINF (-260)

#define yypact_value_is_default(Yyn) \
  ((Yyn) == YYPACT_NINF)

#define YYTABLE_NINF (-78)

#define yytable_value_is_error(Yyn) \
  0

  /* YYPACT[STATE-NUM] -- Index in YYTABLE of the portion describing
     STATE-NUM.  */
static const yytype_int16 yypact[] =
{
     107,    11,    10,   101,   141,    75,    70,  -260,   107,  -260,
    -260,  -260,  -260,  -260,   182,  -260,  -260,  -260,  -260,  -260,
    -260,  -260,  -260,    10,   101,  -260,  -260,  -260,    10,    47,
      32,    39,   101,    10,  -260,  -260,  -260,  -260,  -260,  -260,
    -260,  -260,  -260,  -260,    98,   112,   127,   139,   164,   203,
     176,   206,   207,   169,   169,    47,     3,  -260,  -260,  -260,
       3,  -260,    47,  -260,  -260,    47,    47,   138,    -2,   161,
     162,   134,    30,  -260,  -260,   147,   155,  -260,  -260,  -260,
     -19,  -260,   169,   -19,  -260,   123,  -260,  -260,  -260,  -260,
    -260,  -260,  -260,  -260,  -260,  -260,   210,  -260,  -260,  -260,
    -260,    47,    47,    47,  -260,  -260,  -260,  -260,  -260,  -260,
    -260,  -260,   205,  -260,  -260,   104,   104,  -260,  -260,   208,
    -260,  -260,  -260,   209,   167,   168,   213,  -260,   147,   211,
     212,  -260,   214,     5,   215,   216,   217,  -260,   155,  -260,
    -260,  -260,  -260,  -260,  -260,   221,  -260,  -260,   128,   225,
     226,   227,  -260,  -260,  -260,   228,   170,  -260,  -260,   104,
     -10,   222,   223,  -260,  -260,   -14,   184,    34,   173,   174,
     175,    15,  -260,  -260,   234,   236,    65,  -260,   229,    -6,
     230,   238,  -260,   128,  -260,    90,   102,   121,    88,  -260,
     198,   237,  -260,  -260,    10,    -9,     4,  -260,  -260,   218,
    -260,  -260,  -260,   232,  -260,   196,   197,  -260,   239,    58,
     131,   240,  -260,   241,   242,  -260,   244,  -260,    65,    -6,
      50,    50,  -260,  -260,  -260,   243,   246,   247,  -260,   251,
    -260,    90,  -260,   248,   250,   253,  -260,   258,  -260,   102,
    -260,   254,   255,   256,  -260,   265,  -260,   121,  -260,   259,
     260,   261,   263,  -260,   271,  -260,    88,   201,  -260,   104,
    -260,  -260,  -260,  -260,  -260,    -5,  -260,   219,  -260,  -260,
      15,   266,  -260,   268,   272,  -260,    58,   269,  -260,   274,
     278,  -260,   131,    10,    -6,    50,  -260,  -260,  -260,  -260,
    -260,    -6,   245,    50,  -260,  -260,    -6,    39,    50,  -260,
    -260,    -6,    34,    50,  -260,  -260,    -6,    10,   249,    50,
    -260,  -260,  -260,  -260,  -260,   167,   233,  -260,  -260,   279,
      10,   -19,  -260,  -260,    34,   -19,  -260,  -260,  -260,  -260,
    -260,  -260,  -260,  -260,  -260,  -260,  -260,  -260,  -260,  -260,
    -260,  -260,  -260,  -260,  -260,  -260,    15,  -260,  -260,  -260,
    -260,   280,   231,   281,   235,   282,   220,   264,  -260,  -260
};

  /* YYDEFACT[STATE-NUM] -- Default reduction number in state STATE-NUM.
     Performed when YYTABLE does not specify something else to do.  Zero
     means the default is an error.  */
static const yytype_uint8 yydefact[] =
{
       0,     0,    56,    80,     0,     0,     0,     2,     3,     5,
       6,     7,     8,     9,     0,    57,    58,    54,    82,    83,
      84,    78,    81,     0,     0,   117,   118,   119,     0,   160,
       0,     0,     0,     0,   149,   150,   151,   152,   153,   154,
     155,     1,     4,    11,    20,    22,    24,    26,    28,     0,
      12,     0,     0,   123,   123,   160,   165,   162,   161,   156,
     165,   186,   160,   198,   199,   160,   160,   236,    19,    21,
      23,    25,    27,    10,    13,    60,    86,   128,   127,   126,
     130,   122,   124,   130,   220,     0,   164,   172,   174,   168,
     169,   170,   166,   159,   163,   167,     0,   158,   184,   196,
     209,   160,   160,   160,   231,    37,    38,    36,    39,    14,
      15,    16,    33,    29,    17,     0,     0,    43,    44,    53,
      52,    18,    47,    69,    75,    72,     0,    59,    61,     0,
       0,    96,     0,     0,     0,     0,     0,    85,    87,   138,
     139,   140,   129,   125,   120,     0,   171,   173,   176,     0,
       0,     0,   235,   234,   233,     0,    32,    45,    46,     0,
      68,    74,    71,    55,    62,   102,    98,    97,    99,   100,
     103,   104,    79,    88,     0,     0,   223,   179,     0,     0,
       0,     0,   175,   177,   182,   188,   201,   212,   238,    34,
      31,    51,    50,    76,     0,    73,    70,   109,   108,   114,
      90,    89,    91,     0,    94,     0,     0,    42,     0,   143,
     132,     0,   226,     0,     0,   230,     0,   222,   224,     0,
       0,     0,   157,   178,   191,     0,     0,     0,   195,     0,
     187,   189,   204,     0,     0,     0,   208,     0,   200,   202,
     215,     0,     0,     0,   219,     0,   211,   213,   241,     0,
       0,     0,     0,   246,     0,   237,   239,     0,    30,     0,
      63,    67,    66,    65,    64,   113,    93,   101,    40,    41,
       0,     0,   146,     0,     0,   142,   144,     0,   135,     0,
       0,   131,   133,     0,     0,     0,   221,   225,   180,   183,
     181,     0,     0,     0,   185,   190,     0,     0,     0,   197,
     203,     0,     0,     0,   210,   214,     0,     0,     0,     0,
     232,   240,    35,    49,    48,     0,     0,   112,    92,     0,
       0,     0,   141,   145,     0,     0,   121,   134,   227,   228,
     229,   193,   192,   194,   206,   205,   207,   217,   216,   218,
     244,   242,   243,   245,   110,   111,     0,   148,   147,   137,
     136,     0,   105,     0,   106,     0,   107,   116,   115,    95
};

  /* YYPGOTO[NTERM-NUM].  */
static const yytype_int16 yypgoto[] =
{
    -260,  -260,   283,  -260,  -260,   252,  -260,  -260,  -260,  -260,
    -259,  -118,  -156,  -260,    74,  -260,  -260,   -23,  -260,   165,
    -260,  -260,  -260,  -260,    97,  -165,  -260,   154,  -260,  -260,
    -260,  -260,  -260,  -260,  -260,  -260,   257,   224,  -260,   262,
    -260,    17,  -260,  -197,  -260,  -260,  -260,    25,  -260,  -260,
    -260,  -260,  -260,   -36,   -28,   267,  -213,  -260,  -260,   120,
    -260,  -163,  -260,  -260,  -260,  -260,    73,  -260,  -260,  -260,
       8,  -260,    68,  -260,  -260,  -260,  -260,    61,  -260,  -260,
    -260,  -260,    91,  -260,  -260,  -260,  -260,  -260,    54,  -260
};

  /* YYDEFGOTO[NTERM-NUM].  */
static const yytype_int16 yydefgoto[] =
{
       0,     6,     7,     8,     9,    49,    50,   114,   190,   109,
     208,   119,   120,   121,   122,    10,    51,    17,   126,   127,
     128,   194,    11,    52,    21,    22,   136,   137,   138,   199,
     266,   359,    12,    25,    26,   175,    80,    81,    82,   141,
     280,   281,   282,   142,    27,   174,   274,   275,   276,    13,
      34,    35,    96,    59,    92,    93,    94,    95,   181,   182,
     183,   184,    36,   149,    62,   229,   230,   231,    37,   150,
      65,   237,   238,   239,    38,   151,   245,   246,   247,    39,
     145,   216,   217,   218,    40,   155,   104,   254,   255,   256
};

  /* YYTABLE[YYPACT[STATE-NUM]] -- What to do in state STATE-NUM.  If
     positive, shift that token.  If negative, reduce the rule whose
     number is the opposite.  If YYTABLE_NINF, syntax error.  */
static const yytype_int16 yytable[] =
{
      53,    60,   201,   192,   193,    55,   161,   289,   290,    85,
      67,   319,   197,   215,    14,   261,   168,   315,   316,    84,
     205,   206,   228,   236,   244,   253,    98,    60,   263,    99,
     100,   139,   140,   169,    60,   115,   116,    60,    60,    15,
     117,   191,    86,   -77,    57,    16,   105,   106,    58,   -77,
     107,   108,    87,    57,   264,   215,    85,    58,    88,    89,
      90,    91,   317,    15,   207,   152,   153,   154,   228,    16,
      41,   271,   330,    60,    60,    60,   236,   262,   211,   117,
     333,    61,   272,   118,   244,   336,    56,   351,    28,   212,
     339,   273,    63,   253,    19,    20,   343,    57,    64,    87,
      57,    58,   213,   314,    58,    88,    89,    90,    91,    68,
     179,   214,   248,    29,   224,    30,    31,    32,    33,     1,
       2,    54,     3,    69,   348,   249,   232,   225,   350,    66,
     226,   250,   251,   179,   252,   179,   227,   338,    70,   233,
       4,   313,   198,   234,     5,   240,   277,   179,   235,    18,
      71,   220,   177,   117,    23,   278,    24,   118,   241,   349,
     123,    19,    20,   242,   279,   178,   179,   243,   129,   124,
     125,   260,   146,   179,   180,    72,   130,   101,   147,   131,
     132,   112,   133,   113,   134,   135,    43,   102,   103,   157,
     158,   288,    44,    45,    46,    47,    48,   344,    44,    45,
      46,    47,    48,    77,    78,    79,   257,    73,   258,    75,
      76,   110,   111,   148,   156,   159,   117,   163,   162,   189,
     160,   172,   165,   166,   176,   167,   170,   171,   185,   186,
     187,   188,   200,   195,   196,   202,   203,   209,   204,   210,
     219,   221,   222,   267,   259,   268,   269,   270,   286,   265,
     312,   283,   284,   285,   291,   294,   329,   292,   293,   296,
     328,   297,   299,   331,   298,   301,   302,   303,   334,   304,
     306,   307,   308,   337,   309,   310,   322,   320,   340,   321,
     324,   318,   326,   345,   341,   325,   357,   346,   352,   354,
     356,    42,   173,   164,   332,   353,   358,   347,   342,   327,
     355,   323,    74,   223,   295,   335,   143,   300,   305,   287,
     311,    83,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,     0,     0,    97,     0,     0,
       0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
       0,     0,     0,     0,     0,   144
};

static const yytype_int16 yycheck[] =
{
      23,    29,   167,   159,    14,    28,   124,   220,   221,     6,
      33,   270,    26,   176,     3,    24,    11,    22,    23,    55,
       5,     6,   185,   186,   187,   188,    62,    55,    24,    65,
      66,    50,    51,    28,    62,     5,     6,    65,    66,    53,
      49,   159,    39,    53,    50,    59,    48,    49,    54,    59,
      52,    53,    49,    50,    50,   218,     6,    54,    55,    56,
      57,    58,    67,    53,    49,   101,   102,   103,   231,    59,
       0,    13,   285,   101,   102,   103,   239,   195,    13,    49,
     293,    49,    24,    53,   247,   298,    39,   346,    13,    24,
     303,    33,    53,   256,    60,    61,   309,    50,    59,    49,
      50,    54,    37,   259,    54,    55,    56,    57,    58,    11,
      45,    46,    24,    38,    24,    40,    41,    42,    43,    12,
      13,    24,    15,    11,   321,    37,    24,    37,   325,    32,
      40,    43,    44,    45,    46,    45,    46,   302,    11,    37,
      33,   259,   165,    41,    37,    24,    15,    45,    46,    48,
      11,   179,    24,    49,    13,    24,    15,    53,    37,   324,
      13,    60,    61,    42,    33,    37,    45,    46,    13,    22,
      23,   194,    49,    45,    46,    11,    21,    39,    55,    24,
      25,    47,    27,    49,    29,    30,     4,    49,    50,   115,
     116,   219,    16,    17,    18,    19,    20,   315,    16,    17,
      18,    19,    20,    34,    35,    36,     8,     4,    10,     3,
       3,    50,    50,     3,     9,     7,    49,     4,    50,    49,
      11,     4,    11,    11,     3,    11,    11,    11,     3,     3,
       3,     3,    48,    11,    11,    62,    62,     3,    63,     3,
      11,    11,     4,    11,     7,    49,    49,     8,     4,    31,
      49,    11,    11,    11,    11,     4,   284,    11,    11,    11,
     283,    11,     4,   291,    11,    11,    11,    11,   296,     4,
      11,    11,    11,   301,    11,     4,     4,    11,   306,    11,
      11,    62,     4,    50,   307,    11,    66,     8,     8,     8,
       8,     8,   138,   128,    49,    64,    32,   320,    49,   282,
      65,   276,    50,   183,   231,   297,    82,   239,   247,   218,
     256,    54,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    60,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,    -1,
      -1,    -1,    -1,    -1,    -1,    83
};

  /* YYSTOS[STATE-NUM] -- The (internal number of the) accessing
     symbol of state STATE-NUM.  */
static const yytype_uint8 yystos[] =
{
       0,    12,    13,    15,    33,    37,    69,    70,    71,    72,
      83,    90,   100,   117,     3,    53,    59,    85,    48,    60,
      61,    92,    93,    13,    15,   101,   102,   112,    13,    38,
      40,    41,    42,    43,   118,   119,   130,   136,   142,   147,
     152,     0,    70,     4,    16,    17,    18,    19,    20,    73,
      74,    84,    91,    85,    92,    85,    39,    50,    54,   121,
     122,    49,   132,    53,    59,   138,    92,    85,    11,    11,
      11,    11,    11,     4,    73,     3,     3,    34,    35,    36,
     104,   105,   106,   104,   121,     6,    39,    49,    55,    56,
      57,    58,   122,   123,   124,   125,   120,   123,   121,   121,
     121,    39,    49,    50,   154,    48,    49,    52,    53,    77,
      50,    50,    47,    49,    75,     5,     6,    49,    53,    79,
      80,    81,    82,    13,    22,    23,    86,    87,    88,    13,
      21,    24,    25,    27,    29,    30,    94,    95,    96,    50,
      51,   107,   111,   105,   107,   148,    49,    55,     3,   131,
     137,   143,   121,   121,   121,   153,     9,    82,    82,     7,
      11,    79,    50,     4,    87,    11,    11,    11,    11,    28,
      11,    11,     4,    95,   113,   103,     3,    24,    37,    45,
      46,   126,   127,   128,   129,     3,     3,     3,     3,    49,
      76,    79,    80,    14,    89,    11,    11,    26,    85,    97,
      48,    93,    62,    62,    63,     5,     6,    49,    78,     3,
       3,    13,    24,    37,    46,   129,   149,   150,   151,    11,
     122,    11,     4,   127,    24,    37,    40,    46,   129,   133,
     134,   135,    24,    37,    41,    46,   129,   139,   140,   141,
      24,    37,    42,    46,   129,   144,   145,   146,    24,    37,
      43,    44,    46,   129,   155,   156,   157,     8,    10,     7,
      85,    24,    79,    24,    50,    31,    98,    11,    49,    49,
       8,    13,    24,    33,   114,   115,   116,    15,    24,    33,
     108,   109,   110,    11,    11,    11,     4,   150,   122,   124,
     124,    11,    11,    11,     4,   134,    11,    11,    11,     4,
     140,    11,    11,    11,     4,   145,    11,    11,    11,    11,
       4,   156,    49,    79,    80,    22,    23,    67,    62,    78,
      11,    11,     4,   115,    11,    11,     4,   109,    85,   122,
     124,   122,    49,   124,   122,   138,   124,   122,    93,   124,
     122,    85,    49,   124,    79,    50,     8,    85,   111,    93,
     111,    78,     8,    64,     8,    65,     8,    66,    32,    99
};

  /* YYR1[YYN] -- Symbol number of symbol that rule YYN derives.  */
static const yytype_uint8 yyr1[] =
{
       0,    68,    69,    70,    70,    71,    71,    71,    71,    71,
      72,    72,    73,    73,    74,    74,    74,    74,    74,    74,
      74,    74,    74,    74,    74,    74,    74,    74,    74,    75,
      75,    75,    75,    75,    76,    76,    77,    77,    77,    77,
      78,    78,    78,    79,    80,    81,    81,    81,    82,    82,
      82,    82,    82,    82,    84,    83,    83,    85,    85,    86,
      86,    87,    87,    88,    88,    88,    88,    88,    88,    88,
      88,    88,    88,    88,    88,    88,    89,    89,    91,    90,
      90,    92,    92,    93,    93,    94,    94,    95,    95,    96,
      96,    96,    96,    96,    96,    96,    96,    96,    96,    96,
      96,    96,    96,    96,    96,    96,    96,    96,    97,    97,
      98,    98,    98,    98,    98,    99,    99,   100,   101,   101,
     103,   102,   104,   104,   105,   105,   106,   106,   106,   107,
     107,   108,   108,   109,   109,   110,   110,   110,   111,   111,
     113,   112,   114,   114,   115,   115,   116,   116,   116,   117,
     118,   118,   118,   118,   118,   118,   120,   119,   121,   121,
     121,   122,   122,   123,   123,   123,   124,   124,   124,   124,
     124,   125,   125,   125,   125,   126,   126,   127,   127,   128,
     128,   128,   128,   129,   131,   130,   132,   133,   133,   134,
     134,   135,   135,   135,   135,   135,   137,   136,   138,   138,
     139,   139,   140,   140,   141,   141,   141,   141,   141,   143,
     142,   144,   144,   145,   145,   146,   146,   146,   146,   146,
     148,   147,   149,   149,   150,   150,   151,   151,   151,   151,
     151,   153,   152,   154,   154,   154,   154,   155,   155,   156,
     156,   157,   157,   157,   157,   157,   157
};

  /* YYR2[YYN] -- Number of symbols on the right hand side of rule YYN.  */
static const yytype_int8 yyr2[] =
{
       0,     2,     1,     1,     2,     1,     1,     1,     1,     1,
       4,     3,     1,     2,     3,     3,     3,     3,     3,     2,
       1,     2,     1,     2,     1,     2,     1,     2,     1,     1,
       4,     3,     2,     1,     1,     3,     1,     1,     1,     1,
       2,     2,     1,     1,     1,     2,     2,     1,     5,     5,
       3,     3,     1,     1,     0,     6,     1,     1,     1,     1,
       0,     1,     2,     4,     4,     4,     4,     4,     2,     1,
       3,     2,     1,     3,     2,     1,     1,     0,     0,     6,
       1,     1,     1,     1,     1,     1,     0,     1,     2,     3,
       3,     3,     5,     4,     3,    14,     1,     2,     2,     2,
       2,     4,     2,     2,     2,     8,    10,    12,     1,     1,
       3,     3,     2,     1,     0,     1,     0,     2,     1,     1,
       0,     8,     1,     0,     1,     2,     1,     1,     1,     1,
       0,     1,     0,     1,     2,     1,     3,     3,     1,     1,
       0,     8,     1,     0,     1,     2,     1,     3,     3,     2,
       1,     1,     1,     1,     1,     1,     0,     6,     2,     2,
       0,     1,     1,     1,     1,     0,     1,     1,     1,     1,
       1,     2,     1,     2,     1,     1,     0,     1,     2,     1,
       3,     3,     1,     3,     0,     7,     1,     1,     0,     1,
       2,     1,     3,     3,     3,     1,     0,     7,     1,     1,
       1,     0,     1,     2,     1,     3,     3,     3,     1,     0,
       7,     1,     0,     1,     2,     1,     3,     3,     3,     1,
       0,     7,     1,     0,     1,     2,     1,     3,     3,     3,
       1,     0,     7,     2,     2,     2,     0,     1,     0,     1,
       2,     1,     3,     3,     3,     3,     1
};


enum { YYENOMEM = -2 };

#define yyerrok         (yyerrstatus = 0)
#define yyclearin       (yychar = YYEMPTY)

#define YYACCEPT        goto yyacceptlab
#define YYABORT         goto yyabortlab
#define YYERROR         goto yyerrorlab


#define YYRECOVERING()  (!!yyerrstatus)

#define YYBACKUP(Token, Value)                                    \
  do                                                              \
    if (yychar == YYEMPTY)                                        \
      {                                                           \
        yychar = (Token);                                         \
        yylval = (Value);                                         \
        YYPOPSTACK (yylen);                                       \
        yystate = *yyssp;                                         \
        goto yybackup;                                            \
      }                                                           \
    else                                                          \
      {                                                           \
        yyerror (YY_("syntax error: cannot back up")); \
        YYERROR;                                                  \
      }                                                           \
  while (0)

/* Backward compatibility with an undocumented macro.
   Use YYerror or YYUNDEF. */
#define YYERRCODE YYUNDEF


/* Enable debugging if requested.  */
#if YYDEBUG

# ifndef YYFPRINTF
#  include <stdio.h> /* INFRINGES ON USER NAME SPACE */
#  define YYFPRINTF fprintf
# endif

# define YYDPRINTF(Args)                        \
do {                                            \
  if (yydebug)                                  \
    YYFPRINTF Args;                             \
} while (0)

/* This macro is provided for backward compatibility. */
# ifndef YY_LOCATION_PRINT
#  define YY_LOCATION_PRINT(File, Loc) ((void) 0)
# endif


# define YY_SYMBOL_PRINT(Title, Kind, Value, Location)                    \
do {                                                                      \
  if (yydebug)                                                            \
    {                                                                     \
      YYFPRINTF (stderr, "%s ", Title);                                   \
      yy_symbol_print (stderr,                                            \
                  Kind, Value); \
      YYFPRINTF (stderr, "\n");                                           \
    }                                                                     \
} while (0)


/*-----------------------------------.
| Print this symbol's value on YYO.  |
`-----------------------------------*/

static void
yy_symbol_value_print (FILE *yyo,
                       yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep)
{
  FILE *yyoutput = yyo;
  YY_USE (yyoutput);
  if (!yyvaluep)
    return;
# ifdef YYPRINT
  if (yykind < YYNTOKENS)
    YYPRINT (yyo, yytoknum[yykind], *yyvaluep);
# endif
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  YY_USE (yykind);
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}


/*---------------------------.
| Print this symbol on YYO.  |
`---------------------------*/

static void
yy_symbol_print (FILE *yyo,
                 yysymbol_kind_t yykind, YYSTYPE const * const yyvaluep)
{
  YYFPRINTF (yyo, "%s %s (",
             yykind < YYNTOKENS ? "token" : "nterm", yysymbol_name (yykind));

  yy_symbol_value_print (yyo, yykind, yyvaluep);
  YYFPRINTF (yyo, ")");
}

/*------------------------------------------------------------------.
| yy_stack_print -- Print the state stack from its BOTTOM up to its |
| TOP (included).                                                   |
`------------------------------------------------------------------*/

static void
yy_stack_print (yy_state_t *yybottom, yy_state_t *yytop)
{
  YYFPRINTF (stderr, "Stack now");
  for (; yybottom <= yytop; yybottom++)
    {
      int yybot = *yybottom;
      YYFPRINTF (stderr, " %d", yybot);
    }
  YYFPRINTF (stderr, "\n");
}

# define YY_STACK_PRINT(Bottom, Top)                            \
do {                                                            \
  if (yydebug)                                                  \
    yy_stack_print ((Bottom), (Top));                           \
} while (0)


/*------------------------------------------------.
| Report that the YYRULE is going to be reduced.  |
`------------------------------------------------*/

static void
yy_reduce_print (yy_state_t *yyssp, YYSTYPE *yyvsp,
                 int yyrule)
{
  int yylno = yyrline[yyrule];
  int yynrhs = yyr2[yyrule];
  int yyi;
  YYFPRINTF (stderr, "Reducing stack by rule %d (line %d):\n",
             yyrule - 1, yylno);
  /* The symbols being reduced.  */
  for (yyi = 0; yyi < yynrhs; yyi++)
    {
      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
      yy_symbol_print (stderr,
                       YY_ACCESSING_SYMBOL (+yyssp[yyi + 1 - yynrhs]),
                       &yyvsp[(yyi + 1) - (yynrhs)]);
      YYFPRINTF (stderr, "\n");
    }
}

# define YY_REDUCE_PRINT(Rule)          \
do {                                    \
  if (yydebug)                          \
    yy_reduce_print (yyssp, yyvsp, Rule); \
} while (0)

/* Nonzero means print parse trace.  It is left uninitialized so that
   multiple parsers can coexist.  */
int yydebug;
#else /* !YYDEBUG */
# define YYDPRINTF(Args) ((void) 0)
# define YY_SYMBOL_PRINT(Title, Kind, Value, Location)
# define YY_STACK_PRINT(Bottom, Top)
# define YY_REDUCE_PRINT(Rule)
#endif /* !YYDEBUG */


/* YYINITDEPTH -- initial size of the parser's stacks.  */
#ifndef YYINITDEPTH
# define YYINITDEPTH 200
#endif

/* YYMAXDEPTH -- maximum size the stacks can grow to (effective only
   if the built-in stack extension method is used).

   Do not make this value too large; the results are undefined if
   YYSTACK_ALLOC_MAXIMUM < YYSTACK_BYTES (YYMAXDEPTH)
   evaluated with infinite-precision integer arithmetic.  */

#ifndef YYMAXDEPTH
# define YYMAXDEPTH 10000
#endif






/*-----------------------------------------------.
| Release the memory associated to this symbol.  |
`-----------------------------------------------*/

static void
yydestruct (const char *yymsg,
            yysymbol_kind_t yykind, YYSTYPE *yyvaluep)
{
  YY_USE (yyvaluep);
  if (!yymsg)
    yymsg = "Deleting";
  YY_SYMBOL_PRINT (yymsg, yykind, yyvaluep, yylocationp);

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  YY_USE (yykind);
  YY_IGNORE_MAYBE_UNINITIALIZED_END
}


/* Lookahead token kind.  */
int yychar;

/* The semantic value of the lookahead symbol.  */
YYSTYPE yylval;
/* Number of syntax errors so far.  */
int yynerrs;




/*----------.
| yyparse.  |
`----------*/

int
yyparse (void)
{
    yy_state_fast_t yystate = 0;
    /* Number of tokens to shift before error messages enabled.  */
    int yyerrstatus = 0;

    /* Refer to the stacks through separate pointers, to allow yyoverflow
       to reallocate them elsewhere.  */

    /* Their size.  */
    YYPTRDIFF_T yystacksize = YYINITDEPTH;

    /* The state stack: array, bottom, top.  */
    yy_state_t yyssa[YYINITDEPTH];
    yy_state_t *yyss = yyssa;
    yy_state_t *yyssp = yyss;

    /* The semantic value stack: array, bottom, top.  */
    YYSTYPE yyvsa[YYINITDEPTH];
    YYSTYPE *yyvs = yyvsa;
    YYSTYPE *yyvsp = yyvs;

  int yyn;
  /* The return value of yyparse.  */
  int yyresult;
  /* Lookahead symbol kind.  */
  yysymbol_kind_t yytoken = YYSYMBOL_YYEMPTY;
  /* The variables used to return semantic value and location from the
     action routines.  */
  YYSTYPE yyval;



#define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))

  /* The number of symbols on the RHS of the reduced rule.
     Keep to zero when no symbol should be popped.  */
  int yylen = 0;

  YYDPRINTF ((stderr, "Starting parse\n"));

  yychar = YYEMPTY; /* Cause a token to be read.  */
  goto yysetstate;


/*------------------------------------------------------------.
| yynewstate -- push a new state, which is found in yystate.  |
`------------------------------------------------------------*/
yynewstate:
  /* In all cases, when you get here, the value and location stacks
     have just been pushed.  So pushing a state here evens the stacks.  */
  yyssp++;


/*--------------------------------------------------------------------.
| yysetstate -- set current state (the top of the stack) to yystate.  |
`--------------------------------------------------------------------*/
yysetstate:
  YYDPRINTF ((stderr, "Entering state %d\n", yystate));
  YY_ASSERT (0 <= yystate && yystate < YYNSTATES);
  YY_IGNORE_USELESS_CAST_BEGIN
  *yyssp = YY_CAST (yy_state_t, yystate);
  YY_IGNORE_USELESS_CAST_END
  YY_STACK_PRINT (yyss, yyssp);

  if (yyss + yystacksize - 1 <= yyssp)
#if !defined yyoverflow && !defined YYSTACK_RELOCATE
    goto yyexhaustedlab;
#else
    {
      /* Get the current used size of the three stacks, in elements.  */
      YYPTRDIFF_T yysize = yyssp - yyss + 1;

# if defined yyoverflow
      {
        /* Give user a chance to reallocate the stack.  Use copies of
           these so that the &'s don't force the real ones into
           memory.  */
        yy_state_t *yyss1 = yyss;
        YYSTYPE *yyvs1 = yyvs;

        /* Each stack pointer address is followed by the size of the
           data in use in that stack, in bytes.  This used to be a
           conditional around just the two extra args, but that might
           be undefined if yyoverflow is a macro.  */
        yyoverflow (YY_("memory exhausted"),
                    &yyss1, yysize * YYSIZEOF (*yyssp),
                    &yyvs1, yysize * YYSIZEOF (*yyvsp),
                    &yystacksize);
        yyss = yyss1;
        yyvs = yyvs1;
      }
# else /* defined YYSTACK_RELOCATE */
      /* Extend the stack our own way.  */
      if (YYMAXDEPTH <= yystacksize)
        goto yyexhaustedlab;
      yystacksize *= 2;
      if (YYMAXDEPTH < yystacksize)
        yystacksize = YYMAXDEPTH;

      {
        yy_state_t *yyss1 = yyss;
        union yyalloc *yyptr =
          YY_CAST (union yyalloc *,
                   YYSTACK_ALLOC (YY_CAST (YYSIZE_T, YYSTACK_BYTES (yystacksize))));
        if (! yyptr)
          goto yyexhaustedlab;
        YYSTACK_RELOCATE (yyss_alloc, yyss);
        YYSTACK_RELOCATE (yyvs_alloc, yyvs);
#  undef YYSTACK_RELOCATE
        if (yyss1 != yyssa)
          YYSTACK_FREE (yyss1);
      }
# endif

      yyssp = yyss + yysize - 1;
      yyvsp = yyvs + yysize - 1;

      YY_IGNORE_USELESS_CAST_BEGIN
      YYDPRINTF ((stderr, "Stack size increased to %ld\n",
                  YY_CAST (long, yystacksize)));
      YY_IGNORE_USELESS_CAST_END

      if (yyss + yystacksize - 1 <= yyssp)
        YYABORT;
    }
#endif /* !defined yyoverflow && !defined YYSTACK_RELOCATE */

  if (yystate == YYFINAL)
    YYACCEPT;

  goto yybackup;


/*-----------.
| yybackup.  |
`-----------*/
yybackup:
  /* Do appropriate processing given the current state.  Read a
     lookahead token if we need one and don't already have one.  */

  /* First try to decide what to do without reference to lookahead token.  */
  yyn = yypact[yystate];
  if (yypact_value_is_default (yyn))
    goto yydefault;

  /* Not known => get a lookahead token if don't already have one.  */

  /* YYCHAR is either empty, or end-of-input, or a valid lookahead.  */
  if (yychar == YYEMPTY)
    {
      YYDPRINTF ((stderr, "Reading a token\n"));
      yychar = yylex ();
    }

  if (yychar <= YYEOF)
    {
      yychar = YYEOF;
      yytoken = YYSYMBOL_YYEOF;
      YYDPRINTF ((stderr, "Now at end of input.\n"));
    }
  else if (yychar == YYerror)
    {
      /* The scanner already issued an error message, process directly
         to error recovery.  But do not keep the error token as
         lookahead, it is too special and may lead us to an endless
         loop in error recovery. */
      yychar = YYUNDEF;
      yytoken = YYSYMBOL_YYerror;
      goto yyerrlab1;
    }
  else
    {
      yytoken = YYTRANSLATE (yychar);
      YY_SYMBOL_PRINT ("Next token is", yytoken, &yylval, &yylloc);
    }

  /* If the proper action on seeing token YYTOKEN is to reduce or to
     detect an error, take that action.  */
  yyn += yytoken;
  if (yyn < 0 || YYLAST < yyn || yycheck[yyn] != yytoken)
    goto yydefault;
  yyn = yytable[yyn];
  if (yyn <= 0)
    {
      if (yytable_value_is_error (yyn))
        goto yyerrlab;
      yyn = -yyn;
      goto yyreduce;
    }

  /* Count tokens shifted since error; after three, turn off error
     status.  */
  if (yyerrstatus)
    yyerrstatus--;

  /* Shift the lookahead token.  */
  YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
  yystate = yyn;
  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END

  /* Discard the shifted token.  */
  yychar = YYEMPTY;
  goto yynewstate;


/*-----------------------------------------------------------.
| yydefault -- do the default action for the current state.  |
`-----------------------------------------------------------*/
yydefault:
  yyn = yydefact[yystate];
  if (yyn == 0)
    goto yyerrlab;
  goto yyreduce;


/*-----------------------------.
| yyreduce -- do a reduction.  |
`-----------------------------*/
yyreduce:
  /* yyn is the number of a rule to reduce with.  */
  yylen = yyr2[yyn];

  /* If YYLEN is nonzero, implement the default value of the action:
     '$$ = $1'.

     Otherwise, the following line sets YYVAL to garbage.
     This behavior is undocumented and Bison
     users should not rely upon it.  Assigning to YYVAL
     unconditionally makes the parser a bit smaller, and it avoids a
     GCC warning that YYVAL may be used uninitialized.  */
  yyval = yyvsp[1-yylen];


  YY_REDUCE_PRINT (yyn);
  switch (yyn)
    {
  case 14: /* globalopt: TOK_HOSTNAME TOK_ASSIGN hname  */
#line 923 "gram.y"
                    {
			if (global.flags & GLOBAL_CHANGE_HOSTNAME) {
			    pmsprintf(mess, sizeof(mess), "Duplicate global hostname clause");
			    yyerror(mess);
			}
			if (inarch.label.hostname != NULL && strcmp(inarch.label.hostname, (yyvsp[0].str)) == 0) {
			    /* no change ... */
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Global hostname (%s): No change", inarch.label.hostname);
				yywarn(mess);
			    }
			    free((yyvsp[0].str));
			}
			else {
			    global.hostname = (yyvsp[0].str);
			    global.flags |= GLOBAL_CHANGE_HOSTNAME;
			}
		    }
#line 2343 "gram.tab.c"
    break;

  case 15: /* globalopt: TOK_TIMEZONE TOK_ASSIGN TOK_STRING  */
#line 942 "gram.y"
                    {
			if (global.flags & GLOBAL_CHANGE_TIMEZONE) {
			    pmsprintf(mess, sizeof(mess), "Duplicate global timezone clause");
			    yyerror(mess);
			}
			if (inarch.label.timezone != NULL && strcmp(inarch.label.timezone, (yyvsp[0].str)) == 0) {
			    /* no change ... */
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Global timezone (%s): No change", inarch.label.timezone);
				yywarn(mess);
			    }
			    free((yyvsp[0].str));
			}
			else {
			    global.timezone =  (yyvsp[0].str);
			    global.flags |= GLOBAL_CHANGE_TIMEZONE;
			}
		    }
#line 2366 "gram.tab.c"
    break;

  case 16: /* globalopt: TOK_ZONEINFO TOK_ASSIGN TOK_STRING  */
#line 961 "gram.y"
                    {
			if (global.flags & GLOBAL_CHANGE_ZONEINFO) {
			    pmsprintf(mess, sizeof(mess), "Duplicate global zoneinfo clause");
			    yyerror(mess);
			}
			if (outarch.version < PM_LOG_VERS03) {
			    pmsprintf(mess, sizeof(mess), "Global zoneinfo clause requires output archive version of 3 or greater");
			    yyerror(mess);
			}
			if (inarch.label.zoneinfo != NULL && strcmp(inarch.label.zoneinfo, (yyvsp[0].str)) == 0) {
			    /* no change ... */
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Global zoneinfo (%s): No change", inarch.label.zoneinfo);
				yywarn(mess);
			    }
			    free((yyvsp[0].str));
			}
			else {
			    global.zoneinfo = (yyvsp[0].str);
			    global.flags |= GLOBAL_CHANGE_ZONEINFO;
			}
		    }
#line 2393 "gram.tab.c"
    break;

  case 17: /* globalopt: TOK_FEATURES TOK_ASSIGN feature_bits  */
#line 984 "gram.y"
                    {
			if (global.flags & GLOBAL_CHANGE_FEATURES) {
			    pmsprintf(mess, sizeof(mess), "Duplicate global features clause");
			    yyerror(mess);
			}
			if (outarch.version < PM_LOG_VERS03) {
			    pmsprintf(mess, sizeof(mess), "Global features clause requires output archive version of 3 or greater");
			    yyerror(mess);
			}
			if (inarch.label.features == (yyvsp[0].uival)) {
			    /* no change ... */
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Global features (%d): No change", inarch.label.features);
				yywarn(mess);
			    }
			}
			else {
			    global.features = (yyvsp[0].uival);
			    global.flags |= GLOBAL_CHANGE_FEATURES;
			}
		    }
#line 2419 "gram.tab.c"
    break;

  case 18: /* globalopt: TOK_TIME TOK_ASSIGN signtime  */
#line 1006 "gram.y"
                    {
			if (global.flags & GLOBAL_CHANGE_TIME) {
			    pmsprintf(mess, sizeof(mess), "Duplicate global time clause");
			    yyerror(mess);
			}
			if (global.time.sec == 0 && global.time.nsec == 0) {
			    /* no change ... */
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Global time: No change");
				yywarn(mess);
			    }
			}
			else
			    global.flags |= GLOBAL_CHANGE_TIME;
		    }
#line 2439 "gram.tab.c"
    break;

  case 19: /* globalopt: TOK_HOSTNAME TOK_ASSIGN  */
#line 1022 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting hostname in hostname clause");
			yyerror(mess);
		    }
#line 2448 "gram.tab.c"
    break;

  case 20: /* globalopt: TOK_HOSTNAME  */
#line 1027 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in hostname clause");
			yyerror(mess);
		    }
#line 2457 "gram.tab.c"
    break;

  case 21: /* globalopt: TOK_TIMEZONE TOK_ASSIGN  */
#line 1032 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting timezone string in timezone clause");
			yyerror(mess);
		    }
#line 2466 "gram.tab.c"
    break;

  case 22: /* globalopt: TOK_TIMEZONE  */
#line 1037 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in timezone clause");
			yyerror(mess);
		    }
#line 2475 "gram.tab.c"
    break;

  case 23: /* globalopt: TOK_ZONEINFO TOK_ASSIGN  */
#line 1042 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting zoneinfo string in zoneinfo clause");
			yyerror(mess);
		    }
#line 2484 "gram.tab.c"
    break;

  case 24: /* globalopt: TOK_ZONEINFO  */
#line 1047 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in zoneinfo clause");
			yyerror(mess);
		    }
#line 2493 "gram.tab.c"
    break;

  case 25: /* globalopt: TOK_FEATURES TOK_ASSIGN  */
#line 1052 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting features value in features clause");
			yyerror(mess);
		    }
#line 2502 "gram.tab.c"
    break;

  case 26: /* globalopt: TOK_FEATURES  */
#line 1057 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in features clause");
			yyerror(mess);
		    }
#line 2511 "gram.tab.c"
    break;

  case 27: /* globalopt: TOK_TIME TOK_ASSIGN  */
#line 1062 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting delta of the form [+-][HH:[MM:]]SS[.d...] in time clause");
			yyerror(mess);
		    }
#line 2520 "gram.tab.c"
    break;

  case 28: /* globalopt: TOK_TIME  */
#line 1067 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in time clause");
			yyerror(mess);
		    }
#line 2529 "gram.tab.c"
    break;

  case 29: /* feature_bits: TOK_NUMBER  */
#line 1074 "gram.y"
                    {
			char	*q;
			(yyval.uival) = (unsigned int)strtoul((yyvsp[0].str), &q, 10);
			free((yyvsp[0].str));
		    }
#line 2539 "gram.tab.c"
    break;

  case 30: /* feature_bits: TOK_BITS TOK_LPAREN bits_list TOK_RPAREN  */
#line 1080 "gram.y"
                    {
			(yyval.uival) = (yyvsp[-1].uival);
		    }
#line 2547 "gram.tab.c"
    break;

  case 31: /* feature_bits: TOK_BITS TOK_LPAREN bits_list  */
#line 1084 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting ) after \"bits(\" in features clause");
			yyerror(mess);
		    }
#line 2556 "gram.tab.c"
    break;

  case 32: /* feature_bits: TOK_BITS TOK_LPAREN  */
#line 1089 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting list of bit numbers after \"bits(\" in features clause");
			yyerror(mess);
		    }
#line 2565 "gram.tab.c"
    break;

  case 33: /* feature_bits: TOK_BITS  */
#line 1094 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting ( after \"bits\" in features clause");
			yyerror(mess);
		    }
#line 2574 "gram.tab.c"
    break;

  case 34: /* bits_list: TOK_NUMBER  */
#line 1101 "gram.y"
                    {
			int	b;
			b = atoi((yyvsp[0].str));
			if (b < 0 || b > 31) {
			    pmsprintf(mess, sizeof(mess), "Bit \"%s\" not in the range 0..31", (yyvsp[0].str));
			    yyerror(mess);
			}
			(yyval.uival) = (unsigned int)1 << b;
			free((yyvsp[0].str));
		    }
#line 2589 "gram.tab.c"
    break;

  case 35: /* bits_list: bits_list TOK_COMMA TOK_NUMBER  */
#line 1112 "gram.y"
                    {
			int	b;
			b = atoi((yyvsp[0].str));
			if (b < 0 || b > 31) {
			    pmsprintf(mess, sizeof(mess), "Bit \"%s\" not in the range 0..31", (yyvsp[0].str));
			    yyerror(mess);
			}
			(yyval.uival) = (unsigned int)(yyvsp[-2].uival) | ((unsigned int)1 << b);
			free((yyvsp[0].str));
		    }
#line 2604 "gram.tab.c"
    break;

  case 40: /* signnumber: TOK_PLUS TOK_NUMBER  */
#line 1138 "gram.y"
                    {
			(yyval.ival) = atoi((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2613 "gram.tab.c"
    break;

  case 41: /* signnumber: TOK_MINUS TOK_NUMBER  */
#line 1143 "gram.y"
                    {
			(yyval.ival) = -atoi((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2622 "gram.tab.c"
    break;

  case 42: /* signnumber: TOK_NUMBER  */
#line 1148 "gram.y"
                    {
			(yyval.ival) = atoi((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2631 "gram.tab.c"
    break;

  case 43: /* number: TOK_NUMBER  */
#line 1155 "gram.y"
                    {
			(yyval.ival) = atoi((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2640 "gram.tab.c"
    break;

  case 44: /* float: TOK_FLOAT  */
#line 1162 "gram.y"
                    {
			(yyval.dval) = atof((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 2649 "gram.tab.c"
    break;

  case 46: /* signtime: TOK_MINUS time  */
#line 1169 "gram.y"
                                 { global.time.sec = -global.time.sec; }
#line 2655 "gram.tab.c"
    break;

  case 48: /* time: number TOK_COLON number TOK_COLON float  */
#line 1174 "gram.y"
                    { 
			if ((yyvsp[-2].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Minutes (%d) in time clause more than 59", (yyvsp[-2].ival));
			    yywarn(mess);
			}
			if ((yyvsp[0].dval) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%.6f) in time clause more than 59", (yyvsp[0].dval));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[-4].ival) * 3600 + (yyvsp[-2].ival) * 60 + (int)(yyvsp[0].dval);
			global.time.nsec = (int)(1000000000*(((yyvsp[0].dval) - (int)(yyvsp[0].dval)))+0.5);
		    }
#line 2672 "gram.tab.c"
    break;

  case 49: /* time: number TOK_COLON number TOK_COLON number  */
#line 1187 "gram.y"
                    { 
			if ((yyvsp[-2].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Minutes (%d) in time clause more than 59", (yyvsp[-2].ival));
			    yywarn(mess);
			}
			if ((yyvsp[0].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%d) in time clause more than 59", (yyvsp[0].ival));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[-4].ival) * 3600 + (yyvsp[-2].ival) * 60 + (yyvsp[0].ival);
		    }
#line 2688 "gram.tab.c"
    break;

  case 50: /* time: number TOK_COLON float  */
#line 1199 "gram.y"
                    { 
			if ((yyvsp[-2].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Minutes (%d) in time clause more than 59", (yyvsp[-2].ival));
			    yywarn(mess);
			}
			if ((yyvsp[0].dval) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%.6f) in time clause more than 59", (yyvsp[0].dval));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[-2].ival) * 60 + (int)(yyvsp[0].dval);
			global.time.nsec = (int)(1000000000*(((yyvsp[0].dval) - (int)(yyvsp[0].dval)))+0.5);
		    }
#line 2705 "gram.tab.c"
    break;

  case 51: /* time: number TOK_COLON number  */
#line 1212 "gram.y"
                    { 
			if ((yyvsp[-2].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Minutes (%d) in time clause more than 59", (yyvsp[-2].ival));
			    yywarn(mess);
			}
			if ((yyvsp[0].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%d) in time clause more than 59", (yyvsp[0].ival));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[-2].ival) * 60 + (yyvsp[0].ival);
		    }
#line 2721 "gram.tab.c"
    break;

  case 52: /* time: float  */
#line 1224 "gram.y"
                    {
			if ((yyvsp[0].dval) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%.6f) in time clause more than 59", (yyvsp[0].dval));
			    yywarn(mess);
			}
			global.time.sec = (int)(yyvsp[0].dval);
			global.time.nsec = (int)(1000000000*(((yyvsp[0].dval) - (int)(yyvsp[0].dval)))+0.5);
		    }
#line 2734 "gram.tab.c"
    break;

  case 53: /* time: number  */
#line 1233 "gram.y"
                    {
			if ((yyvsp[0].ival) > 59) {
			    pmsprintf(mess, sizeof(mess), "Seconds (%d) in time clause more than 59", (yyvsp[0].ival));
			    yywarn(mess);
			}
			global.time.sec = (yyvsp[0].ival);
			global.time.nsec = 0;
		    }
#line 2747 "gram.tab.c"
    break;

  case 54: /* $@1: %empty  */
#line 1244 "gram.y"
                    {
			if (current_star_indom) {
			    __pmContext		*ctxp;
			    __pmHashCtl		*hcp;
			    __pmHashNode	*node;

			    ctxp = __pmHandleToPtr(pmWhichContext());
			    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			    PM_UNLOCK(ctxp->c_lock);
			    hcp = &ctxp->c_archctl->ac_log->hashindom;
			    star_domain = pmInDom_domain((yyvsp[0].indom));
			    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
				 node != NULL;
				 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
				if (pmInDom_domain((pmInDom)(node->key)) == star_domain)
				    current_indomspec = start_indom((pmInDom)(node->key));
			    }
			    do_walk_indom = 1;
			}
			else {
			    current_indomspec = start_indom((yyvsp[0].indom));
			    do_walk_indom = 0;
			}
		    }
#line 2783 "gram.tab.c"
    break;

  case 56: /* indomspec: TOK_INDOM  */
#line 1277 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting <domain>.<serial> or <domain>.* in indom rule");
			yyerror(mess);
		    }
#line 2792 "gram.tab.c"
    break;

  case 57: /* indom_int: TOK_FLOAT  */
#line 1284 "gram.y"
                    {
			int		domain;
			int		serial;
			int		sts;
			sts = sscanf((yyvsp[0].str), "%d.%d", &domain, &serial);
			if (sts < 2) {
			    pmsprintf(mess, sizeof(mess), "Missing serial field for indom");
			    yyerror(mess);
			}
			if (domain < 1 || domain >= DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for indom", domain);
			    yyerror(mess);
			}
			if (serial < 0 || serial >= 4194304) {
			    pmsprintf(mess, sizeof(mess), "Illegal serial field (%d) for indom", serial);
			    yyerror(mess);
			}
			current_star_indom = 0;
			free((yyvsp[0].str));
			(yyval.indom) = pmInDom_build(domain, serial);
		    }
#line 2818 "gram.tab.c"
    break;

  case 58: /* indom_int: TOK_INDOM_STAR  */
#line 1306 "gram.y"
                    {
			int		domain;
			sscanf((yyvsp[0].str), "%d.", &domain);
			if (domain < 1 || domain >= DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for indom", domain);
			    yyerror(mess);
			}
			current_star_indom = 1;
			free((yyvsp[0].str));
			(yyval.indom) = pmInDom_build(domain, 0);
		    }
#line 2834 "gram.tab.c"
    break;

  case 63: /* indomopt: TOK_INDOM TOK_ASSIGN duplicateopt indom_int  */
#line 1328 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    pmInDom	indom;
			    if (indom_root->new_indom != indom_root->old_indom) {
				pmsprintf(mess, sizeof(mess), "Duplicate indom clause for indom %s", pmInDomStr(indom_root->old_indom));
				yyerror(mess);
			    }
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[0].indom)), pmInDom_serial(ip->old_indom));
			    else
				indom = (yyvsp[0].indom);
			    if (indom != ip->old_indom)
				ip->new_indom = indom;
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Instance domain %s: indom: No change", pmInDomStr(ip->old_indom));
				    yywarn(mess);
				}
			    }
			    ip->indom_flags |= (yyvsp[-1].ival);
			}
		    }
#line 2863 "gram.tab.c"
    break;

  case 64: /* indomopt: TOK_INAME TOK_STRING TOK_ASSIGN TOK_STRING  */
#line 1353 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    if (change_inst_by_name(ip->old_indom, (yyvsp[-2].str), (yyvsp[0].str)) < 0)
				yyerror(mess);
			}
			free((yyvsp[-2].str));
			/* Note: $4 referenced from new_iname[] */
		    }
#line 2877 "gram.tab.c"
    break;

  case 65: /* indomopt: TOK_INAME TOK_STRING TOK_ASSIGN TOK_DELETE  */
#line 1363 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    if (change_inst_by_name(ip->old_indom, (yyvsp[-2].str), NULL) < 0)
			    	yyerror(mess);
			}
			free((yyvsp[-2].str));
		    }
#line 2890 "gram.tab.c"
    break;

  case 66: /* indomopt: TOK_INST number TOK_ASSIGN number  */
#line 1372 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    if (change_inst_by_inst(ip->old_indom, (yyvsp[-2].ival), (yyvsp[0].ival)) < 0)
				yyerror(mess);
			}
		    }
#line 2902 "gram.tab.c"
    break;

  case 67: /* indomopt: TOK_INST number TOK_ASSIGN TOK_DELETE  */
#line 1380 "gram.y"
                    {
			indomspec_t	*ip;
			for (ip = walk_indom(W_START); ip != NULL; ip = walk_indom(W_NEXT)) {
			    if (change_inst_by_inst(ip->old_indom, (yyvsp[-2].ival), PM_IN_NULL) < 0)
				yyerror(mess);
			}
		    }
#line 2914 "gram.tab.c"
    break;

  case 68: /* indomopt: TOK_INDOM TOK_ASSIGN  */
#line 1388 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting <domain>.<serial> or <domain>.* in indom clause");
			yyerror(mess);
		    }
#line 2923 "gram.tab.c"
    break;

  case 69: /* indomopt: TOK_INDOM  */
#line 1393 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in indom clause");
			yyerror(mess);
		    }
#line 2932 "gram.tab.c"
    break;

  case 70: /* indomopt: TOK_INAME TOK_STRING TOK_ASSIGN  */
#line 1398 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting new external instance name string or DELETE in iname clause");
			yyerror(mess);
		    }
#line 2941 "gram.tab.c"
    break;

  case 71: /* indomopt: TOK_INAME TOK_STRING  */
#line 1403 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in iname clause");
			yyerror(mess);
		    }
#line 2950 "gram.tab.c"
    break;

  case 72: /* indomopt: TOK_INAME  */
#line 1408 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting old external instance name string in iname clause");
			yyerror(mess);
		    }
#line 2959 "gram.tab.c"
    break;

  case 73: /* indomopt: TOK_INST number TOK_ASSIGN  */
#line 1413 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting new internal instance identifier or DELETE in inst clause");
			yyerror(mess);
		    }
#line 2968 "gram.tab.c"
    break;

  case 74: /* indomopt: TOK_INST number  */
#line 1418 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting -> in inst clause");
			yyerror(mess);
		    }
#line 2977 "gram.tab.c"
    break;

  case 75: /* indomopt: TOK_INST  */
#line 1423 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting old internal instance identifier in inst clause");
			yyerror(mess);
		    }
#line 2986 "gram.tab.c"
    break;

  case 76: /* duplicateopt: TOK_DUPLICATE  */
#line 1429 "gram.y"
                                        { (yyval.ival) = INDOM_DUPLICATE; }
#line 2992 "gram.tab.c"
    break;

  case 77: /* duplicateopt: %empty  */
#line 1430 "gram.y"
                                        { (yyval.ival) = 0; }
#line 2998 "gram.tab.c"
    break;

  case 78: /* $@2: %empty  */
#line 1434 "gram.y"
                    {
			if (current_star_metric) {
			    __pmContext		*ctxp;
			    __pmHashCtl		*hcp;
			    __pmHashNode	*node;

			    ctxp = __pmHandleToPtr(pmWhichContext());
			    assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			    PM_UNLOCK(ctxp->c_lock);
			    hcp = &ctxp->c_archctl->ac_log->hashpmid;
			    star_domain = pmID_domain((yyvsp[0].pmid));
			    if (current_star_metric == 1)
				star_cluster = pmID_cluster((yyvsp[0].pmid));
			    else
				star_cluster = PM_ID_NULL;
			    for (node = __pmHashWalk(hcp, PM_HASH_WALK_START);
				 node != NULL;
				 node = __pmHashWalk(hcp, PM_HASH_WALK_NEXT)) {
				if (pmID_domain((pmID)(node->key)) == star_domain &&
				    (star_cluster == PM_ID_NULL ||
				     star_cluster == pmID_cluster((pmID)(node->key))))
				    current_metricspec = start_metric((pmID)(node->key));
			    }
			    do_walk_metric = 1;
			}
			else {
			    if ((yyvsp[0].pmid) == PM_ID_NULL)
				/* metric not in archive */
				current_metricspec = NULL;
			    else
				current_metricspec = start_metric((yyvsp[0].pmid));
			    do_walk_metric = 0;
			}
		    }
#line 3044 "gram.tab.c"
    break;

  case 80: /* metricspec: TOK_METRIC  */
#line 1477 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting metric name or <domain>.<cluster>.<item> or <domain>.<cluster>.* or <domain>.*.* in metric rule");
			yyerror(mess);
		    }
#line 3053 "gram.tab.c"
    break;

  case 82: /* pmid_or_name: TOK_GNAME  */
#line 1485 "gram.y"
                    {
			int	sts;
			pmID	pmid;
			sts = pmLookupName(1, (const char **)&(yyvsp[0].str), &pmid);
			if (sts < 0) {
			    if (wflag) {
				pmsprintf(mess, sizeof(mess), "Metric: %s: %s", (yyvsp[0].str), pmErrStr(sts));
				yywarn(mess);
			    }
			    pmid = PM_ID_NULL;
			}
			current_star_metric = 0;
			free((yyvsp[0].str));
			(yyval.pmid) = pmid;
		    }
#line 3073 "gram.tab.c"
    break;

  case 83: /* pmid_int: TOK_PMID_INT  */
#line 1503 "gram.y"
                    {
			int	domain;
			int	cluster;
			int	item;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d.%d.%d", &domain, &cluster, &item);
			assert(sts == 3);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			if (cluster < 0 || cluster >= 4096) {
			    pmsprintf(mess, sizeof(mess), "Illegal cluster field (%d) for pmid", cluster);
			    yyerror(mess);
			}
			if (item < 0 || item >= 1024) {
			    pmsprintf(mess, sizeof(mess), "Illegal item field (%d) for pmid", item);
			    yyerror(mess);
			}
			current_star_metric = 0;
			free((yyvsp[0].str));
			(yyval.pmid) = pmID_build(domain, cluster, item);
		    }
#line 3105 "gram.tab.c"
    break;

  case 84: /* pmid_int: TOK_PMID_STAR  */
#line 1531 "gram.y"
                    {
			int	domain;
			int	cluster;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d.%d.", &domain, &cluster);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			if (sts == 2) {
			    if (cluster >= 4096) {
				pmsprintf(mess, sizeof(mess), "Illegal cluster field (%d) for pmid", cluster);
				yyerror(mess);
			    }
			    current_star_metric = 1;
			}
			else {
			    cluster = 0;
			    current_star_metric = 2;
			}
			free((yyvsp[0].str));
			(yyval.pmid) = pmID_build(domain, cluster, 0);
		    }
#line 3137 "gram.tab.c"
    break;

  case 89: /* metricopt: TOK_PMID TOK_ASSIGN pmid_int  */
#line 1569 "gram.y"
                    {
			metricspec_t	*mp;
			pmID		pmid;
			for (mp = walk_metric(W_START, METRIC_CHANGE_PMID, "pmid", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_PMID, "pmid", 0)) {
			    if (current_star_metric == 1)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster((yyvsp[0].pmid)), pmID_item(mp->old_desc.pmid));
			    else if (current_star_metric == 2)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster(mp->old_desc.pmid), pmID_item(mp->old_desc.pmid));
			    else
				pmid = (yyvsp[0].pmid);
			    if (pmid == mp->old_desc.pmid) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): pmid: No change", mp->old_name, pmIDStr(mp->old_desc.pmid));
				    yywarn(mess);
				}
			    }
			    else {
				mp->new_desc.pmid = pmid;
				mp->flags |= METRIC_CHANGE_PMID;
			    }
			}
		    }
#line 3165 "gram.tab.c"
    break;

  case 90: /* metricopt: TOK_NAME TOK_ASSIGN TOK_GNAME  */
#line 1593 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_NAME, "name", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_NAME, "name", 0)) {
			    if (strcmp((yyvsp[0].str), mp->old_name) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): name: No change", mp->old_name, pmIDStr(mp->old_desc.pmid));
				    yywarn(mess);
				}
			    }
			    else {
				int	sts;
				pmID	pmid;
				sts = pmLookupName(1, (const char **)&(yyvsp[0].str), &pmid);
				if (sts >= 0) {
				    pmsprintf(mess, sizeof(mess), "Metric name %s already assigned for PMID %s", (yyvsp[0].str), pmIDStr(pmid));
				    yyerror(mess);
				}
				mp->new_name = (yyvsp[0].str);
				mp->flags |= METRIC_CHANGE_NAME;
			    }
			}
		    }
#line 3193 "gram.tab.c"
    break;

  case 91: /* metricopt: TOK_TYPE TOK_ASSIGN TOK_TYPE_NAME  */
#line 1617 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_TYPE, "type", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_TYPE, "type", 0)) {
			    if ((yyvsp[0].ival) == mp->old_desc.type) {
				/* old == new, so no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): type: PM_TYPE_%s: No change", mp->old_name, pmIDStr(mp->old_desc.pmid), pmTypeStr(mp->old_desc.type));
				    yywarn(mess);
				}
			    }
			    else {
				if (mp->old_desc.type == PM_TYPE_32 ||
				    mp->old_desc.type == PM_TYPE_U32 ||
				    mp->old_desc.type == PM_TYPE_64 ||
				    mp->old_desc.type == PM_TYPE_U64 ||
				    mp->old_desc.type == PM_TYPE_FLOAT ||
				    mp->old_desc.type == PM_TYPE_DOUBLE) {
				    mp->new_desc.type = (yyvsp[0].ival);
				    mp->flags |= METRIC_CHANGE_TYPE;
				}
				else {
				    pmsprintf(mess, sizeof(mess), "Old type (PM_TYPE_%s) must be numeric", pmTypeStr(mp->old_desc.type));
				    yyerror(mess);
				}
			    }
			}
		    }
#line 3225 "gram.tab.c"
    break;

  case 92: /* metricopt: TOK_TYPE TOK_IF TOK_TYPE_NAME TOK_ASSIGN TOK_TYPE_NAME  */
#line 1645 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_TYPE, "type", 1); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_TYPE, "type", 1)) {
			    if (mp->old_desc.type != (yyvsp[-2].ival)) {
				if (wflag) {
				    char	tbuf0[20];
				    char	tbuf1[20];
				    pmTypeStr_r(mp->old_desc.type, tbuf0, sizeof(tbuf0));
				    pmTypeStr_r((yyvsp[0].ival), tbuf1, sizeof(tbuf1));
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): type: PM_TYPE_%s: No conditional change to PM_TYPE_%s", mp->old_name, pmIDStr(mp->old_desc.pmid), tbuf0, tbuf1);
				    yywarn(mess);
				}
			    }
			    else {
				if (mp->old_desc.type == PM_TYPE_32 ||
				    mp->old_desc.type == PM_TYPE_U32 ||
				    mp->old_desc.type == PM_TYPE_64 ||
				    mp->old_desc.type == PM_TYPE_U64 ||
				    mp->old_desc.type == PM_TYPE_FLOAT ||
				    mp->old_desc.type == PM_TYPE_DOUBLE) {
				    mp->new_desc.type = (yyvsp[0].ival);
				    mp->flags |= METRIC_CHANGE_TYPE;
				}
				else {
				    pmsprintf(mess, sizeof(mess), "Old type (PM_TYPE_%s) must be numeric", pmTypeStr(mp->old_desc.type));
				    yyerror(mess);
				}
			    }
			}
		    }
#line 3260 "gram.tab.c"
    break;

  case 93: /* metricopt: TOK_INDOM TOK_ASSIGN null_or_indom pick  */
#line 1676 "gram.y"
                    {
			metricspec_t	*mp;
			pmInDom		indom;
			for (mp = walk_metric(W_START, METRIC_CHANGE_INDOM, "indom", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_INDOM, "indom", 0)) {
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[-1].indom)), pmInDom_serial(mp->old_desc.indom));
			    else
				indom = (yyvsp[-1].indom);
			    if (indom == mp->old_desc.indom) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): indom: %s: No change", mp->old_name, pmIDStr(mp->old_desc.pmid), pmInDomStr(mp->old_desc.indom));
				    yywarn(mess);
				}
			    }
			    else {
				if ((output == OUTPUT_MIN ||
					  output == OUTPUT_MAX ||
					  output == OUTPUT_SUM ||
					  output == OUTPUT_AVG) &&
					 mp->old_desc.type != PM_TYPE_32 &&
					 mp->old_desc.type != PM_TYPE_U32 &&
					 mp->old_desc.type != PM_TYPE_64 &&
					 mp->old_desc.type != PM_TYPE_U64 &&
					 mp->old_desc.type != PM_TYPE_FLOAT &&
					 mp->old_desc.type != PM_TYPE_DOUBLE) {
				    pmsprintf(mess, sizeof(mess), "OUTPUT option MIN, MAX, AVG or SUM requires type to be numeric, not PM_TYPE_%s", pmTypeStr(mp->old_desc.type));
				    yyerror(mess);
				}
				mp->new_desc.indom = indom;
				mp->flags |= METRIC_CHANGE_INDOM;
				mp->output = output;
				if (output == OUTPUT_ONE) {
				    mp->one_name = one_name;
				    mp->one_inst = one_inst;
				    if (mp->old_desc.indom == PM_INDOM_NULL)
					/*
					 * singular input, pick first (only)
					 * value, not one_inst matching ...
					 * one_inst used for output instance
					 * id
					 */
					mp->output = OUTPUT_FIRST;
				}
				if (output == OUTPUT_ALL) {
				    /*
				     * No OUTPUT clause, set up the defaults
				     * based on indom types:
				     * non-NULL -> NULL
				     *		OUTPUT_FIRST, inst PM_IN_NULL
				     * NULL -> non-NULL
				     *		OUTPUT_FIRST, inst 0
				     * non-NULL -> non-NULL
				     * 		all instances selected
				     *		(nothing to do for defaults)
				     * NULL -> NULL
				     *		caught above in no change case
				     */
				    if (mp->old_desc.indom != PM_INDOM_NULL &&
				        mp->new_desc.indom == PM_INDOM_NULL) {
					mp->output = OUTPUT_FIRST;
					mp->one_inst = PM_IN_NULL;
				    }
				    else if (mp->old_desc.indom == PM_INDOM_NULL &&
				             mp->new_desc.indom != PM_INDOM_NULL) {
					mp->output = OUTPUT_FIRST;
					mp->one_inst = 0;
				    }
				}
			    }
			}
			output = OUTPUT_ALL;	/* for next time */
		    }
#line 3338 "gram.tab.c"
    break;

  case 94: /* metricopt: TOK_SEM TOK_ASSIGN TOK_SEM_NAME  */
#line 1750 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_SEM, "sem", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_SEM, "sem", 0)) {
			    if ((yyvsp[0].ival) == mp->old_desc.sem) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): sem: %s: No change", mp->old_name, pmIDStr(mp->old_desc.pmid), SemStr(mp->old_desc.sem));
				    yywarn(mess);
				}
			    }
			    else {
				mp->new_desc.sem = (yyvsp[0].ival);
				mp->flags |= METRIC_CHANGE_SEM;
			    }
			}
		    }
#line 3359 "gram.tab.c"
    break;

  case 95: /* metricopt: TOK_UNITS TOK_ASSIGN signnumber TOK_COMMA signnumber TOK_COMMA signnumber TOK_COMMA TOK_SPACE_NAME TOK_COMMA TOK_TIME_NAME TOK_COMMA TOK_COUNT_NAME rescaleopt  */
#line 1767 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_CHANGE_UNITS, "units", 0); mp != NULL; mp = walk_metric(W_NEXT, METRIC_CHANGE_UNITS, "units", 0)) {
			    if ((yyvsp[-11].ival) == mp->old_desc.units.dimSpace &&
			        (yyvsp[-9].ival) == mp->old_desc.units.dimTime &&
			        (yyvsp[-7].ival) == mp->old_desc.units.dimCount &&
			        (yyvsp[-5].ival) == mp->old_desc.units.scaleSpace &&
			        (yyvsp[-3].ival) == mp->old_desc.units.scaleTime &&
			        (yyvsp[-1].ival) == mp->old_desc.units.scaleCount) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): units: %s: No change", mp->old_name, pmIDStr(mp->old_desc.pmid), pmUnitsStr(&mp->old_desc.units));
				    yywarn(mess);
				}
			    }
			    else {
				mp->new_desc.units.dimSpace = (yyvsp[-11].ival);
				mp->new_desc.units.dimTime = (yyvsp[-9].ival);
				mp->new_desc.units.dimCount = (yyvsp[-7].ival);
				mp->new_desc.units.scaleSpace = (yyvsp[-5].ival);
				mp->new_desc.units.scaleTime = (yyvsp[-3].ival);
				mp->new_desc.units.scaleCount = (yyvsp[-1].ival);
				mp->flags |= METRIC_CHANGE_UNITS;
				if ((yyvsp[0].ival) == 1) {
				    if ((yyvsp[-11].ival) == mp->old_desc.units.dimSpace &&
					(yyvsp[-9].ival) == mp->old_desc.units.dimTime &&
					(yyvsp[-7].ival) == mp->old_desc.units.dimCount)
					/* OK, no dim changes */
					mp->flags |= METRIC_RESCALE;
				    else {
					if (wflag) {
					    pmsprintf(mess, sizeof(mess), "Metric: %s (%s): Dimension changed, cannot rescale", mp->old_name, pmIDStr(mp->old_desc.pmid));
					    yywarn(mess);
					}
				    }
				}
				else if (sflag) {
				    if ((yyvsp[-11].ival) == mp->old_desc.units.dimSpace &&
					(yyvsp[-9].ival) == mp->old_desc.units.dimTime &&
					(yyvsp[-7].ival) == mp->old_desc.units.dimCount)
					mp->flags |= METRIC_RESCALE;
				}
			    }
			}
		    }
#line 3409 "gram.tab.c"
    break;

  case 96: /* metricopt: TOK_DELETE  */
#line 1813 "gram.y"
                    {
			metricspec_t	*mp;
			for (mp = walk_metric(W_START, METRIC_DELETE, "delete", 1); mp != NULL; mp = walk_metric(W_NEXT, METRIC_DELETE, "delete", 0)) {
			    mp->flags |= METRIC_DELETE;
			}
		    }
#line 3420 "gram.tab.c"
    break;

  case 97: /* metricopt: TOK_PMID TOK_ASSIGN  */
#line 1820 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting <domain>.<cluster>.<item> or <domain>.<cluster>.* or <domain>.*.* in pmid clause");
			yyerror(mess);
		    }
#line 3429 "gram.tab.c"
    break;

  case 98: /* metricopt: TOK_NAME TOK_ASSIGN  */
#line 1825 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting metric name in iname clause");
			yyerror(mess);
		    }
#line 3438 "gram.tab.c"
    break;

  case 99: /* metricopt: TOK_TYPE TOK_ASSIGN  */
#line 1830 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting XXX (from PM_TYPE_XXX) in type clause");
			yyerror(mess);
		    }
#line 3447 "gram.tab.c"
    break;

  case 100: /* metricopt: TOK_TYPE TOK_IF  */
#line 1835 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting XXX (from PM_TYPE_XXX) after if in type clause");
			yyerror(mess);
		    }
#line 3456 "gram.tab.c"
    break;

  case 101: /* metricopt: TOK_TYPE TOK_IF TOK_TYPE_NAME TOK_ASSIGN  */
#line 1840 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting XXX (from PM_TYPE_XXX) in type clause");
			yyerror(mess);
		    }
#line 3465 "gram.tab.c"
    break;

  case 102: /* metricopt: TOK_INDOM TOK_ASSIGN  */
#line 1845 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting <domain>.<serial> or NULL in indom clause");
			yyerror(mess);
		    }
#line 3474 "gram.tab.c"
    break;

  case 103: /* metricopt: TOK_SEM TOK_ASSIGN  */
#line 1850 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting XXX (from PM_SEM_XXX) in sem clause");
			yyerror(mess);
		    }
#line 3483 "gram.tab.c"
    break;

  case 104: /* metricopt: TOK_UNITS TOK_ASSIGN  */
#line 1855 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting 3 numeric values for dim* fields of units");
			yyerror(mess);
		    }
#line 3492 "gram.tab.c"
    break;

  case 105: /* metricopt: TOK_UNITS TOK_ASSIGN signnumber TOK_COMMA signnumber TOK_COMMA signnumber TOK_COMMA  */
#line 1860 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting 0 or XXX (from PM_SPACE_XXX) for scaleSpace field of units");
			yyerror(mess);
		    }
#line 3501 "gram.tab.c"
    break;

  case 106: /* metricopt: TOK_UNITS TOK_ASSIGN signnumber TOK_COMMA signnumber TOK_COMMA signnumber TOK_COMMA TOK_SPACE_NAME TOK_COMMA  */
#line 1865 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting 0 or XXX (from PM_TIME_XXX) for scaleTime field of units");
			yyerror(mess);
		    }
#line 3510 "gram.tab.c"
    break;

  case 107: /* metricopt: TOK_UNITS TOK_ASSIGN signnumber TOK_COMMA signnumber TOK_COMMA signnumber TOK_COMMA TOK_SPACE_NAME TOK_COMMA TOK_TIME_NAME TOK_COMMA  */
#line 1870 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting 0 or ONE for scaleCount field of units");
			yyerror(mess);
		    }
#line 3519 "gram.tab.c"
    break;

  case 109: /* null_or_indom: TOK_NULL_INT  */
#line 1878 "gram.y"
                    {
			(yyval.indom) = PM_INDOM_NULL;
		    }
#line 3527 "gram.tab.c"
    break;

  case 110: /* pick: TOK_OUTPUT TOK_INST number  */
#line 1884 "gram.y"
                    {
			output = OUTPUT_ONE;
			one_inst = (yyvsp[0].ival);
			one_name = NULL;
		    }
#line 3537 "gram.tab.c"
    break;

  case 111: /* pick: TOK_OUTPUT TOK_INAME TOK_STRING  */
#line 1890 "gram.y"
                    {
			output = OUTPUT_ONE;
			one_inst = PM_IN_NULL;
			one_name = (yyvsp[0].str);
		    }
#line 3547 "gram.tab.c"
    break;

  case 112: /* pick: TOK_OUTPUT TOK_OUTPUT_TYPE  */
#line 1896 "gram.y"
                    {
			output = (yyvsp[0].ival);
		    }
#line 3555 "gram.tab.c"
    break;

  case 113: /* pick: TOK_OUTPUT  */
#line 1900 "gram.y"
                    {
			pmsprintf(mess, sizeof(mess), "Expecting FIRST or LAST or INST or INAME or MIN or MAX or AVG for OUTPUT instance option");
			yyerror(mess);
		    }
#line 3564 "gram.tab.c"
    break;

  case 115: /* rescaleopt: TOK_RESCALE  */
#line 1907 "gram.y"
                              { (yyval.ival) = 1; }
#line 3570 "gram.tab.c"
    break;

  case 116: /* rescaleopt: %empty  */
#line 1909 "gram.y"
                    { (yyval.ival) = 0; }
#line 3576 "gram.tab.c"
    break;

  case 120: /* $@3: %empty  */
#line 1920 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		target_types;
			int		this_type;
			int		type;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_textspec = NULL;
			if ((yyvsp[-2].pmid) == PM_ID_NULL) {
			    /* Metric referenced by name is not in the archive */
			    do_walk_text = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_metric) {
				/* Set up for metrics specified using globbing. */
				star_domain = pmID_domain((yyvsp[-2].pmid));
				if (current_star_metric == 1)
				    star_cluster = pmID_cluster((yyvsp[-2].pmid));
				else
				    star_cluster = PM_ID_NULL;
			    }

			    /* We're looking for text of the specified type(s) for metric(s). */
			    target_types = PM_TEXT_PMID | (yyvsp[-1].ival);
			    hcp1 = &ctxp->c_archctl->ac_log->hashtext;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if ((this_type & target_types) != this_type)
				    continue;

				/*
				 * Collect the text records associated with the specified
				 * metric(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_metric) {
					/* Match the globbed metric spec and keep looking. */
					if (pmID_domain((pmID)(node2->key)) == star_domain &&
					    (star_cluster == PM_ID_NULL ||
					     star_cluster == pmID_cluster((pmID)(node2->key)))) {
					    current_textspec = start_text(this_type, (pmID)(node2->key), (yyvsp[0].str));
					    if (current_textspec) {
						current_textspec->flags |= TEXT_ACTIVE;
						++found;
					    }

					}
				    }
				    else {
					/* Match the exact metric PMID. */
					if ((pmID)(node2->key) == (yyvsp[-2].pmid)) {
					    current_textspec = start_text(this_type, (pmID)(node2->key), (yyvsp[0].str));
					    if (current_textspec) {
						current_textspec->flags |= TEXT_ACTIVE;
						++found;
					    }
					    /*
					     * Stop looking if we have found all of the specified
					     * types.
					     */
					    type = this_type & PM_TEXT_TYPE_MASK;
					    target_types ^= type;
					    if ((target_types & PM_TEXT_TYPE_MASK) == 0)
						break;
					}
				    }
				}
				if ((target_types & PM_TEXT_TYPE_MASK) == 0)
				    break;
			    }
			    do_walk_text = (found > 1);
			}
		    }
#line 3677 "gram.tab.c"
    break;

  case 122: /* opttexttypes: texttypes  */
#line 2020 "gram.y"
                    { (yyval.ival) = (yyvsp[0].ival); }
#line 3683 "gram.tab.c"
    break;

  case 123: /* opttexttypes: %empty  */
#line 2022 "gram.y"
                    { (yyval.ival) = PM_TEXT_ONELINE; }
#line 3689 "gram.tab.c"
    break;

  case 124: /* texttypes: texttype  */
#line 2026 "gram.y"
                    { (yyval.ival) = (yyvsp[0].ival); }
#line 3695 "gram.tab.c"
    break;

  case 125: /* texttypes: texttype texttypes  */
#line 2028 "gram.y"
                    { (yyval.ival) = (yyvsp[-1].ival) | (yyvsp[0].ival); }
#line 3701 "gram.tab.c"
    break;

  case 126: /* texttype: TOK_TEXT_STAR  */
#line 2032 "gram.y"
                    { (yyval.ival) = PM_TEXT_ONELINE | PM_TEXT_HELP; }
#line 3707 "gram.tab.c"
    break;

  case 127: /* texttype: TOK_HELP  */
#line 2034 "gram.y"
                    { (yyval.ival) = PM_TEXT_HELP; }
#line 3713 "gram.tab.c"
    break;

  case 128: /* texttype: TOK_ONELINE  */
#line 2036 "gram.y"
                    { (yyval.ival) = PM_TEXT_ONELINE; }
#line 3719 "gram.tab.c"
    break;

  case 129: /* opttextvalue: textstring  */
#line 2040 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3725 "gram.tab.c"
    break;

  case 130: /* opttextvalue: %empty  */
#line 2042 "gram.y"
                    { (yyval.str) = NULL; }
#line 3731 "gram.tab.c"
    break;

  case 132: /* opttextmetricoptlist: %empty  */
#line 2047 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_ACTIVE, "active", 0); tp != NULL; tp = walk_text(W_NEXT, 0, "", 0)) {
			    tp->flags &= ~TEXT_ACTIVE;
			}
		    }
#line 3742 "gram.tab.c"
    break;

  case 135: /* textmetricopt: TOK_DELETE  */
#line 2060 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_DELETE, "delete", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_DELETE, "delete", 0)) {
			    tp->flags |= TEXT_DELETE;
			}
		    }
#line 3753 "gram.tab.c"
    break;

  case 136: /* textmetricopt: TOK_TEXT TOK_ASSIGN textstring  */
#line 2067 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_CHANGE_TEXT, "text", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_CHANGE_TEXT, "text", 0)) {
			    if (tp->new_text) {
				if (strcmp(tp->new_text, (yyvsp[0].str)) == 0) {
				    pmsprintf(mess, sizeof(mess), "Duplicate text change clause");
				    yyerror(mess);
				}
				else {
				    pmsprintf(mess, sizeof(mess), "Conflicting text change clause");
				    yyerror(mess);
				}
				free((yyvsp[0].str));
			    }
			    else if (strcmp(tp->old_text, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Help text: No change");
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				tp->new_text = (yyvsp[0].str);
				tp->flags |= TEXT_CHANGE_TEXT;
			    }
			}
		    }
#line 3786 "gram.tab.c"
    break;

  case 137: /* textmetricopt: TOK_METRIC TOK_ASSIGN pmid_int  */
#line 2096 "gram.y"
                    {
			textspec_t	*tp;
			pmID		pmid;
			for (tp = walk_text(W_START, TEXT_CHANGE_ID, "id", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_CHANGE_ID, "id", 0)) {
			    if (current_star_metric == 1)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster((yyvsp[0].pmid)), pmID_item(tp->old_id));
			    else if (current_star_metric == 2)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster(tp->old_id), pmID_item(tp->old_id));
			    else
				pmid = (yyvsp[0].pmid);
			    if (pmid == tp->old_id) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Text for metric: %s: metric: No change", pmIDStr(tp->old_id));
				    yywarn(mess);
				}
			    }
			    else {
				const textspec_t *tp1;
				/* Warn if the target metric already has text of this type. */
				/*
				 * Search all of the change specs for others targeting
				 * the same metric.
				 */
				for (tp1 = text_root; tp1 != NULL; tp1 = tp1->t_next) {
				    if (! (tp1->flags & TEXT_CHANGE_ID))
					continue; /* not an id change spec */
				    if (tp1->new_type != tp->new_type)
					continue; /* target is a different type */
				    if (tp1 == tp)
					continue; /* same spec */
				    if (tp1->new_id == pmid) {
					/* conflict with another change spec. */
					pmsprintf(mess, sizeof(mess), "Text for metric: %s: metric: Conflicting metric change", pmIDStr(tp->old_id));
					yyerror(mess);
					break;
				    }
				}
				if (tp1 == NULL) {
				    /* No conflict */
				    tp->new_id = pmid;
				    tp->flags |= TEXT_CHANGE_ID;
				}
			    }
			}
		    }
#line 3837 "gram.tab.c"
    break;

  case 138: /* textstring: TOK_STRING  */
#line 2145 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3843 "gram.tab.c"
    break;

  case 139: /* textstring: TOK_TEXT_STRING  */
#line 2147 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 3849 "gram.tab.c"
    break;

  case 140: /* $@4: %empty  */
#line 2151 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		target_types;
			int		this_type;
			int		type;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_textspec = NULL;
			if ((yyvsp[-2].indom) == PM_ID_NULL) {
			    /* Indom is not in the archive */
			    do_walk_text = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_indom) {
				/* Set up for indoms specified using globbing. */
				star_domain = pmInDom_domain((yyvsp[-2].indom));
			    }

			    /* We're looking for text of the specified type(s) for indom(s). */
			    target_types = PM_TEXT_INDOM | (yyvsp[-1].ival);
			    hcp1 = &ctxp->c_archctl->ac_log->hashtext;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if ((this_type & target_types) != this_type)
				    continue;

				/*
				 * Collect the text records associated with the specified
				 * metric(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_indom) {
					/* Match the globbed metric spec and keep looking. */
					if (pmInDom_domain((pmID)(node2->key)) == star_domain) {
					    current_textspec = start_text(this_type, (pmID)(node2->key), (yyvsp[0].str));
					    if (current_textspec) {
						current_textspec->flags |= TEXT_ACTIVE;
						++found;
					    }
					}
				    }
				    else {
					/* Match the exact indom id. */
					if ((pmID)(node2->key) == (yyvsp[-2].indom)) {
					    current_textspec = start_text(this_type, (pmID)(node2->key), (yyvsp[0].str));
					    if (current_textspec) {
						current_textspec->flags |= TEXT_ACTIVE;
						++found;
					    }
					    /*
					     * Stop looking if we have found all of the specified
					     * types.
					     */
					    type = this_type & PM_TEXT_TYPE_MASK;
					    target_types ^= type;
					    if ((target_types & PM_TEXT_TYPE_MASK) == 0)
						break;
					}
				    }
				}
				if ((target_types & PM_TEXT_TYPE_MASK) == 0)
				    break;
			    }
			    do_walk_text = (found > 1);
			}
		    }
#line 3943 "gram.tab.c"
    break;

  case 143: /* opttextindomoptlist: %empty  */
#line 2245 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_ACTIVE, "active", 0); tp != NULL; tp = walk_text(W_NEXT, 0, "", 0)) {
			    tp->flags &= ~TEXT_ACTIVE;
			}
		    }
#line 3954 "gram.tab.c"
    break;

  case 146: /* textindomopt: TOK_DELETE  */
#line 2258 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_DELETE, "delete", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_DELETE, "delete", 0)) {
			    tp->flags |= TEXT_DELETE;
			}
		    }
#line 3965 "gram.tab.c"
    break;

  case 147: /* textindomopt: TOK_TEXT TOK_ASSIGN textstring  */
#line 2265 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_CHANGE_TEXT, "text", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_CHANGE_TEXT, "text", 0)) {
			    if (tp->new_text) {
				if (strcmp(tp->new_text, (yyvsp[0].str)) == 0) {
				    pmsprintf(mess, sizeof(mess), "Duplicate text change clause");
				    yyerror(mess);
				}
				else {
				    pmsprintf(mess, sizeof(mess), "Conflicting text change clause");
				    yyerror(mess);
				}
				free((yyvsp[0].str));
			    }
			    else if (strcmp(tp->old_text, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Help text: No change");
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				tp->new_text = (yyvsp[0].str);
				tp->flags |= TEXT_CHANGE_TEXT;
			    }
			}
		    }
#line 3998 "gram.tab.c"
    break;

  case 148: /* textindomopt: TOK_INDOM TOK_ASSIGN indom_int  */
#line 2294 "gram.y"
                    {
			textspec_t	*tp;
			for (tp = walk_text(W_START, TEXT_CHANGE_ID, "id", 0); tp != NULL; tp = walk_text(W_NEXT, TEXT_CHANGE_ID, "id", 0)) {
			    pmInDom	indom;
			    if (tp->new_id != tp->old_id) {
				pmsprintf(mess, sizeof(mess), "Duplicate text clause for indom %s", pmInDomStr(tp->old_id));
				yyerror(mess);
			    }
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[0].indom)), pmInDom_serial(tp->old_id));
			    else
				indom = (yyvsp[0].indom);
			    if (indom != tp->old_id) {
				tp->new_id = indom;
				tp->flags |= TEXT_CHANGE_ID;
			    }
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Text for instance domain %s: indom: No change", pmInDomStr(tp->old_id));
				    yywarn(mess);
				}
			    }
			}
		    }
#line 4028 "gram.tab.c"
    break;

  case 156: /* $@5: %empty  */
#line 2333 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		found = 0;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			/* We're looking for context labels. */
			current_labelspec = NULL;
			hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
			     node1 != NULL;
			     node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
			    __pmHashCtl	*hcp2;
			    __pmHashNode	*node2;
			    int		this_type;

			    /* Was this object type selected? */
			    this_type = (int)(node1->key);
			    if (this_type != PM_LABEL_CONTEXT)
				continue;

			    /*
			     * Collect the label records associated with the specified
			     * metric(s).
			     */
			    hcp2 = (__pmHashCtl *)(node1->data);
			    for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				 node2 != NULL;
				 node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				/* We want all of the context labels. */
				current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
								current_label_name, current_label_value);
				if (current_labelspec) {
				    current_labelspec->flags |= LABEL_ACTIVE;
				    ++found;
				}
			    }
			}
			do_walk_label = found;
		    }
#line 4084 "gram.tab.c"
    break;

  case 158: /* optlabeldetails: jsonname optlabelvalue  */
#line 2388 "gram.y"
                    { current_label_name = (yyvsp[-1].str); }
#line 4090 "gram.tab.c"
    break;

  case 159: /* optlabeldetails: TOK_LABEL_STAR optlabelvalue  */
#line 2390 "gram.y"
                    { current_label_name = NULL; }
#line 4096 "gram.tab.c"
    break;

  case 160: /* optlabeldetails: %empty  */
#line 2392 "gram.y"
                    {
			current_label_name = NULL;
			current_label_value = NULL;
		    }
#line 4105 "gram.tab.c"
    break;

  case 161: /* jsonname: TOK_JSON_STRING  */
#line 2399 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4111 "gram.tab.c"
    break;

  case 162: /* jsonname: TOK_STRING  */
#line 2401 "gram.y"
                    {
			(yyval.str) = add_quotes((yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 4120 "gram.tab.c"
    break;

  case 163: /* optlabelvalue: jsonvalue  */
#line 2408 "gram.y"
                    { current_label_value = (yyvsp[0].str); }
#line 4126 "gram.tab.c"
    break;

  case 164: /* optlabelvalue: TOK_LABEL_STAR  */
#line 2410 "gram.y"
                    { current_label_value = NULL; }
#line 4132 "gram.tab.c"
    break;

  case 165: /* optlabelvalue: %empty  */
#line 2412 "gram.y"
                    { current_label_value = NULL; }
#line 4138 "gram.tab.c"
    break;

  case 166: /* jsonvalue: jsonname  */
#line 2416 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4144 "gram.tab.c"
    break;

  case 167: /* jsonvalue: jsonnumber  */
#line 2418 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4150 "gram.tab.c"
    break;

  case 168: /* jsonvalue: TOK_JSON_TRUE  */
#line 2420 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4156 "gram.tab.c"
    break;

  case 169: /* jsonvalue: TOK_JSON_FALSE  */
#line 2422 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4162 "gram.tab.c"
    break;

  case 170: /* jsonvalue: TOK_JSON_NULL  */
#line 2424 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4168 "gram.tab.c"
    break;

  case 171: /* jsonnumber: TOK_MINUS TOK_NUMBER  */
#line 2428 "gram.y"
                    {
			(yyval.str) = dupcat("-", (yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 4177 "gram.tab.c"
    break;

  case 172: /* jsonnumber: TOK_NUMBER  */
#line 2433 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4183 "gram.tab.c"
    break;

  case 173: /* jsonnumber: TOK_MINUS TOK_JSON_NUMBER  */
#line 2435 "gram.y"
                    {
			(yyval.str) = dupcat("-", (yyvsp[0].str));
			free((yyvsp[0].str));
		    }
#line 4192 "gram.tab.c"
    break;

  case 174: /* jsonnumber: TOK_JSON_NUMBER  */
#line 2440 "gram.y"
                    { (yyval.str) = (yyvsp[0].str); }
#line 4198 "gram.tab.c"
    break;

  case 176: /* optlabelcontextoptlist: %empty  */
#line 2445 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4206 "gram.tab.c"
    break;

  case 179: /* labelcontextopt: TOK_DELETE  */
#line 2455 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4217 "gram.tab.c"
    break;

  case 180: /* labelcontextopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 2462 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for context");
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for context: label: No change");
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4244 "gram.tab.c"
    break;

  case 181: /* labelcontextopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 2486 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for cluster %d.%d", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster %d.%d: value: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4271 "gram.tab.c"
    break;

  case 182: /* labelcontextopt: newlabelspec  */
#line 2510 "gram.y"
                    {
			new_context_label();
		    }
#line 4279 "gram.tab.c"
    break;

  case 183: /* newlabelspec: TOK_NEW jsonname jsonvalue  */
#line 2515 "gram.y"
                    {
			/* The current label name and value must both NOT be specified. */
			if (current_label_name || current_label_value) {
			    pmsprintf(mess, sizeof(mess), "The target label name and value must both not be specified for a NEW label and will be ignored");
			    yywarn(mess);
			    if (current_label_name)
				free(current_label_name);
			    if (current_label_value)
				free(current_label_value);
			}
			current_label_name = (yyvsp[-1].str);
			current_label_value = (yyvsp[0].str);
		    }
#line 4297 "gram.tab.c"
    break;

  case 184: /* $@6: %empty  */
#line 2530 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		 found = 0;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			assert ((yyvsp[-1].ival) != PM_ID_NULL);
			current_label_id = (yyvsp[-1].ival);
			    
			/* We're looking for domain labels. */
			hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
			     node1 != NULL;
			     node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
			    __pmHashCtl	*hcp2;
			    __pmHashNode	*node2;
			    int		this_type;

			    /* Was this object type selected? */
			    this_type = (int)(node1->key);
			    if (this_type != PM_LABEL_DOMAIN)
				continue;

			    /*
			     * Collect the label records associated with the specified
			     * metric domain.
			     */
			    hcp2 = (__pmHashCtl *)(node1->data);
			    for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				 node2 != NULL;
				 node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				/* Match the exact metric domain. */
				if ((pmID)(node2->key) == (yyvsp[-1].ival)) {
				    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
								    current_label_name, current_label_value);
				    if (current_labelspec) {
					current_labelspec->flags |= LABEL_ACTIVE;
					++found;
				    }
				}
			    }
			}
			do_walk_label = found;
		    }
#line 4358 "gram.tab.c"
    break;

  case 186: /* pmid_domain: TOK_NUMBER  */
#line 2590 "gram.y"
                    {
			int	domain;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d", &domain);
			assert(sts == 1);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			current_star_metric = 0;
			free((yyvsp[0].str));
			(yyval.ival) = domain;
		    }
#line 4380 "gram.tab.c"
    break;

  case 188: /* optlabeldomainoptlist: %empty  */
#line 2611 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4388 "gram.tab.c"
    break;

  case 191: /* labeldomainopt: TOK_DELETE  */
#line 2621 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4399 "gram.tab.c"
    break;

  case 192: /* labeldomainopt: TOK_DOMAIN TOK_ASSIGN TOK_NUMBER  */
#line 2628 "gram.y"
                    {
			labelspec_t	*lp;
			int		domain;
			int		sts;
			sts = sscanf((yyvsp[0].str), "%d", &domain);
			assert(sts == 1);
			free((yyvsp[0].str));
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    if (domain == lp->old_id) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for metric domain: %d: No change", lp->old_id);
				    yywarn(mess);
				}
			    }
			    else {
				/*
				 * No need to check for conflicts. Multiple label set records for
				 * the same domain are ok.
				 */
				lp->new_id = domain;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			}
		    }
#line 4429 "gram.tab.c"
    break;

  case 193: /* labeldomainopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 2654 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for domain %d", lp->old_id);
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for domain %d: label: No change", lp->old_id);
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4456 "gram.tab.c"
    break;

  case 194: /* labeldomainopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 2678 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for cluster %d.%d", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster %d.%d: value: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4483 "gram.tab.c"
    break;

  case 195: /* labeldomainopt: newlabelspec  */
#line 2702 "gram.y"
                    {
			new_domain_label(current_label_id);
		    }
#line 4491 "gram.tab.c"
    break;

  case 196: /* $@7: %empty  */
#line 2708 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;
			int		found = 0;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			assert ((yyvsp[-1].ival) != PM_ID_NULL);
			current_label_id = (yyvsp[-1].ival);
			    
			if (current_star_metric) {
			    /* Set up for metrics specified using globbing. */
			    star_domain = pmID_domain((yyvsp[-1].ival));
			    star_cluster = PM_ID_NULL;
			}

			/* We're looking for cluster labels. */
			hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
			     node1 != NULL;
			     node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
			    __pmHashCtl		*hcp2;
			    __pmHashNode	*node2;
			    int			this_type;

			    /* Was this object type selected? */
			    this_type = (int)(node1->key);
			    if (this_type != PM_LABEL_CLUSTER)
				continue;

			    /*
			     * Collect the label records associated with the specified
			     * metric(s).
			     */
			    hcp2 = (__pmHashCtl *)(node1->data);
			    for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				 node2 != NULL;
				 node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				if (current_star_metric) {
				    /* Match the globbed cluster spec and keep looking. */
				    if (pmID_domain((pmID)(node2->key)) == star_domain) {
					current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									current_label_name, current_label_value);
					if (current_labelspec) {
					    current_labelspec->flags |= LABEL_ACTIVE;
					    ++found;
					}
				    }
				}
				else {
				    /* Match the exact cluster spec. */
				    if ((pmID)(node2->key) == (yyvsp[-1].ival)) {
					current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									current_label_name, current_label_value);
					if (current_labelspec) {
					    current_labelspec->flags |= LABEL_ACTIVE;
					    ++found;
					}
				    }
				}
			    }
			}
			do_walk_label = found;
		    }
#line 4571 "gram.tab.c"
    break;

  case 198: /* pmid_cluster: TOK_FLOAT  */
#line 2787 "gram.y"
                    {
			int	domain;
			int	cluster;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d.%d", &domain, &cluster);
			assert(sts == 2);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			if (cluster < 0 || cluster >= 4096) {
			    pmsprintf(mess, sizeof(mess), "Illegal cluster field (%d) for pmid", cluster);
			    yyerror(mess);
			}
			current_star_metric = 0;
			free((yyvsp[0].str));
			(yyval.ival) = pmID_build(domain, cluster, 0);
		    }
#line 4598 "gram.tab.c"
    break;

  case 199: /* pmid_cluster: TOK_INDOM_STAR  */
#line 2810 "gram.y"
                    {
			int	domain;
			int	sts;
			sts = sscanf((yyvsp[0].str), "%d.", &domain);
			assert (sts == 1);
			if (domain < 1 || domain > DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Illegal domain field (%d) for pmid", domain);
			    yyerror(mess);
			}
			else if (domain == DYNAMIC_PMID) {
			    pmsprintf(mess, sizeof(mess), "Dynamic metric domain field (%d) for pmid", domain);
			    yywarn(mess);
			}
			current_star_metric = 1;
			free((yyvsp[0].str));
			(yyval.ival) = pmID_build(domain, 0, 0);
		    }
#line 4620 "gram.tab.c"
    break;

  case 201: /* optlabelclusteroptlist: %empty  */
#line 2831 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4628 "gram.tab.c"
    break;

  case 204: /* labelclusteropt: TOK_DELETE  */
#line 2841 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4639 "gram.tab.c"
    break;

  case 205: /* labelclusteropt: TOK_CLUSTER TOK_ASSIGN pmid_cluster  */
#line 2848 "gram.y"
                    {
			labelspec_t	*lp;
			int		cluster;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    if (current_star_metric)
				cluster = pmID_build(pmID_domain((yyvsp[0].ival)), pmID_cluster(lp->old_id), PM_ID_NULL);
			    else
				cluster = pmID_build(pmID_domain((yyvsp[0].ival)), pmID_cluster((yyvsp[0].ival)), PM_ID_NULL);
			    if (cluster == lp->old_id) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster: %d.%d: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
			    }
			    else {
				/*
				 * No need to check for conflicts. Multiple label set records for
				 * the same cluster are ok.
				 */
				lp->new_id = cluster;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			}
		    }
#line 4669 "gram.tab.c"
    break;

  case 206: /* labelclusteropt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 2874 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for cluster %d.%d", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster %d.%d: label: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4696 "gram.tab.c"
    break;

  case 207: /* labelclusteropt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 2898 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for cluster %d.%d", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for cluster %d.%d: value: No change", pmID_domain(lp->old_id), pmID_cluster(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4723 "gram.tab.c"
    break;

  case 208: /* labelclusteropt: newlabelspec  */
#line 2922 "gram.y"
                    {
			new_cluster_label(current_label_id);
		    }
#line 4731 "gram.tab.c"
    break;

  case 209: /* $@8: %empty  */
#line 2928 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			current_label_id = (yyvsp[-1].pmid);
			if ((yyvsp[-1].pmid) == PM_ID_NULL) {
			    /* Metric referenced by name is not in the archive */
			    do_walk_label = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_metric) {
				/* Set up for metrics specified using globbing. */
				star_domain = pmID_domain((yyvsp[-1].pmid));
				if (current_star_metric == 1)
				    star_cluster = pmID_cluster((yyvsp[-1].pmid));
				else
				    star_cluster = PM_ID_NULL;
			    }

			    /* We're looking for item labels. */
			    hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;
				int		this_type;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if (this_type != PM_LABEL_ITEM)
				    continue;

				/*
				 * Collect the label records associated with the specified
				 * metric(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_metric) {
					/* Match the globbed metric spec and keep looking. */
					if (pmID_domain((pmID)(node2->key)) == star_domain &&
					    (star_cluster == PM_ID_NULL ||
					     star_cluster == pmID_cluster((pmID)(node2->key)))) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }

					}
				    }
				    else {
					/* Match the exact metric PMID. */
					if ((pmID)(node2->key) == (yyvsp[-1].pmid)) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				}
			    }
			    do_walk_label = found;
			}
		    }
#line 4822 "gram.tab.c"
    break;

  case 212: /* optlabelitemoptlist: %empty  */
#line 3019 "gram.y"
                    {
			deactivate_labels();
		    }
#line 4830 "gram.tab.c"
    break;

  case 215: /* labelitemopt: TOK_DELETE  */
#line 3029 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 4841 "gram.tab.c"
    break;

  case 216: /* labelitemopt: TOK_ITEM TOK_ASSIGN pmid_int  */
#line 3036 "gram.y"
                    {
			labelspec_t	*lp;
			pmID		pmid;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    if (current_star_metric == 1)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster((yyvsp[0].pmid)), pmID_item(lp->old_id));
			    else if (current_star_metric == 2)
				pmid = pmID_build(pmID_domain((yyvsp[0].pmid)), pmID_cluster(lp->old_id), pmID_item(lp->old_id));
			    else
				pmid = (yyvsp[0].pmid);
			    if (pmid == lp->old_id) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for metric: %s: item: No change", pmIDStr(lp->old_id));
				    yywarn(mess);
				}
			    }
			    else {
				/*
				 * No need to check for conflicts. Multiple label set records for
				 * the same pmid are ok.
				 */
				lp->new_id = pmid;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			}
		    }
#line 4873 "gram.tab.c"
    break;

  case 217: /* labelitemopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 3064 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for metric %s", pmIDStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for metric %s: label: No change", pmIDStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 4900 "gram.tab.c"
    break;

  case 218: /* labelitemopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 3088 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for metric %s", pmIDStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for metric %s: value: No change", pmIDStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 4927 "gram.tab.c"
    break;

  case 219: /* labelitemopt: newlabelspec  */
#line 3112 "gram.y"
                    {
			new_item_label(current_label_id);
		    }
#line 4935 "gram.tab.c"
    break;

  case 220: /* $@9: %empty  */
#line 3118 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			current_label_id = (yyvsp[-1].indom);
			if ((yyvsp[-1].indom) == PM_ID_NULL) {
			    /* Indom is not in the archive */
			    do_walk_label = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_indom) {
				/* Set up for indoms specified using globbing. */
				star_domain = pmInDom_domain((yyvsp[-1].indom));
			    }

			    /* We're looking for label sets for indom(s). */
			    hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;
				int		this_type;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if (this_type != PM_LABEL_INDOM)
				    continue;

				/*
				 * Collect the label records associated with the specified
				 * imdom(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_indom) {
					/* Match the globbed indom spec and keep looking. */
					if (pmInDom_domain((pmID)(node2->key)) == star_domain) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				    else {
					/* Match the exact indom id. */
					if ((pmID)(node2->key) == (yyvsp[-1].indom)) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key), 0, NULL,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				}
			    }
			    do_walk_label = found;
			}
		    }
#line 5019 "gram.tab.c"
    break;

  case 223: /* optlabelindomoptlist: %empty  */
#line 3202 "gram.y"
                    {
			deactivate_labels();
		    }
#line 5027 "gram.tab.c"
    break;

  case 226: /* labelindomopt: TOK_DELETE  */
#line 3212 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 5038 "gram.tab.c"
    break;

  case 227: /* labelindomopt: TOK_INDOM TOK_ASSIGN indom_int  */
#line 3219 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    pmInDom	indom;
			    if (lp->new_id != lp->old_id) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[0].indom)), pmInDom_serial(lp->old_id));
			    else
				indom = (yyvsp[0].indom);
			    if (indom != lp->old_id) {
				lp->new_id = indom;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s: indom: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
			    }
			}
		    }
#line 5068 "gram.tab.c"
    break;

  case 228: /* labelindomopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 3245 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s: label: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 5095 "gram.tab.c"
    break;

  case 229: /* labelindomopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 3269 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s: value: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 5122 "gram.tab.c"
    break;

  case 230: /* labelindomopt: newlabelspec  */
#line 3293 "gram.y"
                    {
			new_indom_label(current_label_id);
		    }
#line 5130 "gram.tab.c"
    break;

  case 231: /* $@10: %empty  */
#line 3299 "gram.y"
                    {
			__pmContext	*ctxp;
			__pmHashCtl	*hcp1;
			__pmHashNode	*node1;

			ctxp = __pmHandleToPtr(pmWhichContext());
			assert(ctxp != NULL);
    /*
     * Note: This application is single threaded, and once we have ctxp,
     *	     the associated __pmContext will not move and will only be
     *	     accessed or modified synchronously either here or in libpcp.
     *	     We unlock the context so that it can be locked as required
     *	     within libpcp.
     */
			PM_UNLOCK(ctxp->c_lock);
			
			current_labelspec = NULL;
			current_label_id = (yyvsp[-1].indom);
			if ((yyvsp[-1].indom) == PM_ID_NULL) {
			    /* Indom is not in the archive */
			    do_walk_label = 0;
			}
			else {
			    int found = 0;
			    
			    if (current_star_indom) {
				/* Set up for indoms specified using globbing. */
				star_domain = pmInDom_domain((yyvsp[-1].indom));
			    }

			    /* We're looking for label sets for the instances of the indom(s). */
			    hcp1 = &ctxp->c_archctl->ac_log->hashlabels;
			    for (node1 = __pmHashWalk(hcp1, PM_HASH_WALK_START);
				 node1 != NULL;
				 node1 = __pmHashWalk(hcp1, PM_HASH_WALK_NEXT)) {
				__pmHashCtl	*hcp2;
				__pmHashNode	*node2;
				int		this_type;

				/* Was this object type selected? */
				this_type = (int)(node1->key);
				if (this_type != PM_LABEL_INSTANCES)
				    continue;

				/*
				 * Collect the label records associated with the specified
				 * indom instance(s).
				 */
				hcp2 = (__pmHashCtl *)(node1->data);
				for (node2 = __pmHashWalk(hcp2, PM_HASH_WALK_START);
				     node2 != NULL;
				     node2 = __pmHashWalk(hcp2, PM_HASH_WALK_NEXT)) {
				    if (current_star_indom) {
					/* Match the globbed indom spec and keep looking. */
					if (pmInDom_domain((pmID)(node2->key)) == star_domain) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key),
									    current_label_instance, current_label_instance_name,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				    else {
					/* Match the exact indom id. */
					if ((pmID)(node2->key) == (yyvsp[-1].indom)) {
					    current_labelspec = start_label(this_type, (pmID)(node2->key),
									    current_label_instance, current_label_instance_name,
									    current_label_name, current_label_value);
					    if (current_labelspec) {
						current_labelspec->flags |= LABEL_ACTIVE;
						++found;
					    }
					}
				    }
				}
			    }
			    do_walk_label = found;
			}
		    }
#line 5216 "gram.tab.c"
    break;

  case 233: /* optinstancelabeldetails: TOK_STRING optlabeldetails  */
#line 3384 "gram.y"
                    {
			current_label_instance = -1;
			current_label_instance_name = (yyvsp[-1].str);
		    }
#line 5225 "gram.tab.c"
    break;

  case 234: /* optinstancelabeldetails: TOK_NUMBER optlabeldetails  */
#line 3389 "gram.y"
                    {
			current_label_instance = atoi((yyvsp[-1].str));
			free((yyvsp[-1].str));
			current_label_instance_name = NULL;
		    }
#line 5235 "gram.tab.c"
    break;

  case 235: /* optinstancelabeldetails: TOK_LABEL_STAR optlabeldetails  */
#line 3395 "gram.y"
                    {
			current_label_instance = -1;
			current_label_instance_name = NULL;
		    }
#line 5244 "gram.tab.c"
    break;

  case 236: /* optinstancelabeldetails: %empty  */
#line 3400 "gram.y"
                    {
			current_label_instance = -1;
			current_label_instance_name = NULL;
			current_label_name = NULL;
			current_label_value = NULL;
		    }
#line 5255 "gram.tab.c"
    break;

  case 238: /* optlabelinstancesoptlist: %empty  */
#line 3410 "gram.y"
                    {
			deactivate_labels();
		    }
#line 5263 "gram.tab.c"
    break;

  case 241: /* labelinstancesopt: TOK_DELETE  */
#line 3420 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_DELETE, "delete", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_DELETE, "delete", 0)) {
			    lp->flags |= LABEL_DELETE;
			}
		    }
#line 5274 "gram.tab.c"
    break;

  case 242: /* labelinstancesopt: TOK_INSTANCES TOK_ASSIGN indom_int  */
#line 3427 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    pmInDom	indom;
			    if (lp->new_id != lp->old_id) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for instances of indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (current_star_indom)
				indom = pmInDom_build(pmInDom_domain((yyvsp[0].indom)), pmInDom_serial(lp->old_id));
			    else
				indom = (yyvsp[0].indom);
			    if (indom != lp->old_id) {
				lp->new_id = indom;
				lp->flags |= LABEL_CHANGE_ID;
			    }
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s: indom: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
			    }
			}
		    }
#line 5304 "gram.tab.c"
    break;

  case 243: /* labelinstancesopt: TOK_INSTANCE TOK_ASSIGN TOK_NUMBER  */
#line 3453 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_ID, "id", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_ID, "id", 0)) {
			    int	instance;
			    if (lp->new_instance != lp->old_instance) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for instances of indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    instance = atoi((yyvsp[0].str));
			    free((yyvsp[0].str));
			    if (instance != lp->old_instance) {
				lp->new_instance = instance;
				lp->flags |= LABEL_CHANGE_INSTANCE;
			    }
			    else {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instance domain %s instance %d: instance: No change", pmInDomStr(lp->old_id), lp->old_instance);
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			}
		    }
#line 5333 "gram.tab.c"
    break;

  case 244: /* labelinstancesopt: TOK_LABEL TOK_ASSIGN jsonname  */
#line 3478 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_LABEL, "label", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_LABEL, "label", 0)) {
			    if (lp->new_label != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate label clause for instances of indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_label != NULL &&
				strcmp(lp->old_label, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instances of instance domain %s: label: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_label = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_LABEL;
			    }
			}
		    }
#line 5360 "gram.tab.c"
    break;

  case 245: /* labelinstancesopt: TOK_VALUE TOK_ASSIGN jsonvalue  */
#line 3502 "gram.y"
                    {
			labelspec_t	*lp;
			for (lp = walk_label(W_START, LABEL_CHANGE_VALUE, "value", 0); lp != NULL; lp = walk_label(W_NEXT, LABEL_CHANGE_VALUE, "value", 0)) {
			    if (lp->new_value != NULL) {
				pmsprintf(mess, sizeof(mess), "Duplicate value clause for instances of indom %s", pmInDomStr(lp->old_id));
				yyerror(mess);
			    }
			    if (lp->old_value != NULL &&
				strcmp(lp->old_value, (yyvsp[0].str)) == 0) {
				/* no change ... */
				if (wflag) {
				    pmsprintf(mess, sizeof(mess), "Label for instances of instance domain %s: value: No change", pmInDomStr(lp->old_id));
				    yywarn(mess);
				}
				free((yyvsp[0].str));
			    }
			    else {
				lp->new_value = (yyvsp[0].str);
				lp->flags |= LABEL_CHANGE_VALUE;
			    }
			}
		    }
#line 5387 "gram.tab.c"
    break;

  case 246: /* labelinstancesopt: newlabelspec  */
#line 3526 "gram.y"
                    {
			new_indom_instance_label(current_label_id);
		    }
#line 5395 "gram.tab.c"
    break;


#line 5399 "gram.tab.c"

      default: break;
    }
  /* User semantic actions sometimes alter yychar, and that requires
     that yytoken be updated with the new translation.  We take the
     approach of translating immediately before every use of yytoken.
     One alternative is translating here after every semantic action,
     but that translation would be missed if the semantic action invokes
     YYABORT, YYACCEPT, or YYERROR immediately after altering yychar or
     if it invokes YYBACKUP.  In the case of YYABORT or YYACCEPT, an
     incorrect destructor might then be invoked immediately.  In the
     case of YYERROR or YYBACKUP, subsequent parser actions might lead
     to an incorrect destructor call or verbose syntax error message
     before the lookahead is translated.  */
  YY_SYMBOL_PRINT ("-> $$ =", YY_CAST (yysymbol_kind_t, yyr1[yyn]), &yyval, &yyloc);

  YYPOPSTACK (yylen);
  yylen = 0;

  *++yyvsp = yyval;

  /* Now 'shift' the result of the reduction.  Determine what state
     that goes to, based on the state we popped back to and the rule
     number reduced by.  */
  {
    const int yylhs = yyr1[yyn] - YYNTOKENS;
    const int yyi = yypgoto[yylhs] + *yyssp;
    yystate = (0 <= yyi && yyi <= YYLAST && yycheck[yyi] == *yyssp
               ? yytable[yyi]
               : yydefgoto[yylhs]);
  }

  goto yynewstate;


/*--------------------------------------.
| yyerrlab -- here on detecting error.  |
`--------------------------------------*/
yyerrlab:
  /* Make sure we have latest lookahead translation.  See comments at
     user semantic actions for why this is necessary.  */
  yytoken = yychar == YYEMPTY ? YYSYMBOL_YYEMPTY : YYTRANSLATE (yychar);
  /* If not already recovering from an error, report this error.  */
  if (!yyerrstatus)
    {
      ++yynerrs;
      yyerror (YY_("syntax error"));
    }

  if (yyerrstatus == 3)
    {
      /* If just tried and failed to reuse lookahead token after an
         error, discard it.  */

      if (yychar <= YYEOF)
        {
          /* Return failure if at end of input.  */
          if (yychar == YYEOF)
            YYABORT;
        }
      else
        {
          yydestruct ("Error: discarding",
                      yytoken, &yylval);
          yychar = YYEMPTY;
        }
    }

  /* Else will try to reuse lookahead token after shifting the error
     token.  */
  goto yyerrlab1;


/*---------------------------------------------------.
| yyerrorlab -- error raised explicitly by YYERROR.  |
`---------------------------------------------------*/
yyerrorlab:
  /* Pacify compilers when the user code never invokes YYERROR and the
     label yyerrorlab therefore never appears in user code.  */
  if (0)
    YYERROR;

  /* Do not reclaim the symbols of the rule whose action triggered
     this YYERROR.  */
  YYPOPSTACK (yylen);
  yylen = 0;
  YY_STACK_PRINT (yyss, yyssp);
  yystate = *yyssp;
  goto yyerrlab1;


/*-------------------------------------------------------------.
| yyerrlab1 -- common code for both syntax error and YYERROR.  |
`-------------------------------------------------------------*/
yyerrlab1:
  yyerrstatus = 3;      /* Each real token shifted decrements this.  */

  /* Pop stack until we find a state that shifts the error token.  */
  for (;;)
    {
      yyn = yypact[yystate];
      if (!yypact_value_is_default (yyn))
        {
          yyn += YYSYMBOL_YYerror;
          if (0 <= yyn && yyn <= YYLAST && yycheck[yyn] == YYSYMBOL_YYerror)
            {
              yyn = yytable[yyn];
              if (0 < yyn)
                break;
            }
        }

      /* Pop the current state because it cannot handle the error token.  */
      if (yyssp == yyss)
        YYABORT;


      yydestruct ("Error: popping",
                  YY_ACCESSING_SYMBOL (yystate), yyvsp);
      YYPOPSTACK (1);
      yystate = *yyssp;
      YY_STACK_PRINT (yyss, yyssp);
    }

  YY_IGNORE_MAYBE_UNINITIALIZED_BEGIN
  *++yyvsp = yylval;
  YY_IGNORE_MAYBE_UNINITIALIZED_END


  /* Shift the error token.  */
  YY_SYMBOL_PRINT ("Shifting", YY_ACCESSING_SYMBOL (yyn), yyvsp, yylsp);

  yystate = yyn;
  goto yynewstate;


/*-------------------------------------.
| yyacceptlab -- YYACCEPT comes here.  |
`-------------------------------------*/
yyacceptlab:
  yyresult = 0;
  goto yyreturn;


/*-----------------------------------.
| yyabortlab -- YYABORT comes here.  |
`-----------------------------------*/
yyabortlab:
  yyresult = 1;
  goto yyreturn;


#if !defined yyoverflow
/*-------------------------------------------------.
| yyexhaustedlab -- memory exhaustion comes here.  |
`-------------------------------------------------*/
yyexhaustedlab:
  yyerror (YY_("memory exhausted"));
  yyresult = 2;
  goto yyreturn;
#endif


/*-------------------------------------------------------.
| yyreturn -- parsing is finished, clean up and return.  |
`-------------------------------------------------------*/
yyreturn:
  if (yychar != YYEMPTY)
    {
      /* Make sure we have latest lookahead translation.  See comments at
         user semantic actions for why this is necessary.  */
      yytoken = YYTRANSLATE (yychar);
      yydestruct ("Cleanup: discarding lookahead",
                  yytoken, &yylval);
    }
  /* Do not reclaim the symbols of the rule whose action triggered
     this YYABORT or YYACCEPT.  */
  YYPOPSTACK (yylen);
  YY_STACK_PRINT (yyss, yyssp);
  while (yyssp != yyss)
    {
      yydestruct ("Cleanup: popping",
                  YY_ACCESSING_SYMBOL (+*yyssp), yyvsp);
      YYPOPSTACK (1);
    }
#ifndef yyoverflow
  if (yyss != yyssa)
    YYSTACK_FREE (yyss);
#endif

  return yyresult;
}

#line 3531 "gram.y"

